import pdb
#pdb.set_trace()
import pickle
from collections import defaultdict
from argparse import ArgumentParser
import os
import sys


def Check_contig_in_HCbk_dict(HCbk_dict,PS_flag_1,PS_flag_2):
    break_flag = 0
    for key,val in HCbk_dict.items():
        PS_flag_1_curr = key[0]
        PS_flag_2_curr = key[1]
        if PS_flag_1 >= PS_flag_1_curr and PS_flag_2 <= PS_flag_2_curr:
            use_PS_flag_1 = PS_flag_1_curr
            use_PS_flag_2 = PS_flag_2_curr
            break_flag = 1
            break
    if break_flag == 1:
        return (use_PS_flag_1,use_PS_flag_2)
    else:
        return (0,0)


def Make_supercontig_based_on_HCbk(HCbk_file,contig_sam_file,ref_fasta_dict,use_chr_num):
    merge_dict = defaultdict(lambda: defaultdict(list))
    merge_len = defaultdict(list)
    merge_contig_num = defaultdict(int)
    f = open(contig_sam_file,"rb")
    HCbk_dict = pickle.load(open(HCbk_file,"rb"))
    curr = 0
    for line in f:
        curr += 1
        data = line.decode().rsplit()
        if data[0][0] != "@":
            mapq = int(data[4])
            chr_num = data[2]
            if chr_num == "chrX":
                chr_num = "chr23"
            cigar = data[5]
            if "H" in cigar:
                H_clip_flag = 1
            else:
                H_clip_flag = 0
            if mapq >= 20 and chr_num == use_chr_num and H_clip_flag == 0:
                data_split = data[0].split("_")
                contig_num = int(data_split[0])
                PS_flag_1 = int(data_split[1].split("PS")[1].split(":")[0])
                PS_flag_2 = int(data_split[1].split("PS")[1].split(":")[1])
                HP_flag = data_split[2]
                locus = int(data[3])
                contig_seq = data[9]
                use_PS_flag_1,use_PS_flag_2 = Check_contig_in_HCbk_dict(HCbk_dict,PS_flag_1,PS_flag_2)
                if use_PS_flag_1 != 0 and use_PS_flag_2 != 0:
                    merge_contig_num[contig_num] = 1
                    merge_dict[(use_PS_flag_1,use_PS_flag_2,HP_flag)][contig_num] = [contig_num,chr_num,contig_seq, str(PS_flag_1) + ":" + str(PS_flag_2) + "_" + HP_flag]

    count_1 = 0
    count_2 = 0
    count_3 = 0
    count_supercontig = 1
    count_no_supercontig = 0
    supercontig_dict = defaultdict(list)
    remove_contig_dict = defaultdict(int)
    use_for_supercontig_dict = defaultdict(int)
    ### For every big phase block, starting merging
    for key, contig_info_dict in merge_dict.items():
        max_contig_num = sorted(contig_info_dict.keys())[-1]
        new_contig_num = max_contig_num
        HP_flag = key[2]
        HCbk_list = HCbk_dict[(key[0],key[1])]
        for HC_bk in HCbk_list[1:-1]:
            HC_bk_string = ref_fasta_dict[HC_bk:(HC_bk+20)]
            idx_all = []
            contig_merge = []
            for contig_num,one_contig in contig_info_dict.items():
                break_flag = 0
                contig_seq = one_contig[2]
                try:
                    idx = contig_seq.index(HC_bk_string)
                    idx_all.append(idx)
                    contig_merge.append(one_contig)
                except:
                    pass

            idx_dict = defaultdict(int)
            contig_num_dict = defaultdict(int)
            contig_seq_dict = defaultdict(int)
            contig_PS_dict = defaultdict(str)
            before_bk_len = defaultdict(int)
            after_bk_len = defaultdict(int)
            if len(contig_merge) == 2:
                count = 0
                print("----")
                for one_contig in contig_merge:
                    idx_dict[count] = idx_all[count]
                    contig_num_dict[count] = one_contig[0]
                    contig_seq_dict[count] = one_contig[2]
                    contig_PS_dict[count] = one_contig[-1]
                    before_bk_len[count] = len(contig_seq_dict[count][:idx_dict[count]])
                    after_bk_len[count] = len(contig_seq_dict[count][idx_dict[count]:])
                    #print(contig_num,len(contig_seq),idx,before_bk_len,after_bk_len)
                    #print(contig_seq[idx:idx+50])
                    count += 1 
                count_2 += 1

                ### Make Supercontig 
                if before_bk_len[0] < before_bk_len[1] and after_bk_len[0] > after_bk_len[1]:
                    before_bk_string = contig_seq_dict[1][:idx_dict[1]]
                    after_bk_string = contig_seq_dict[0][idx_dict[0]:]
                    supercontig = before_bk_string + after_bk_string
                    if contig_num_dict[0] <= max_contig_num:
                        use_for_supercontig_dict[contig_num_dict[0]] = 1
                    if contig_num_dict[1] <= max_contig_num:
                        use_for_supercontig_dict[contig_num_dict[1]] = 1
                    del contig_info_dict[contig_num_dict[0]]
                    del contig_info_dict[contig_num_dict[1]]
                    new_contig_num += 1
                    PS_str = contig_PS_dict[0] + "-" + contig_PS_dict[1]
                    contig_info_dict[new_contig_num] = [new_contig_num,chr_num,supercontig,PS_str]
                    print(len(contig_seq_dict[0]),len(contig_seq_dict[1]),len(supercontig))
                elif before_bk_len[1] < before_bk_len[0] and after_bk_len[1] > after_bk_len[0]:
                    before_bk_string = contig_seq_dict[0][:idx_dict[0]]
                    after_bk_string = contig_seq_dict[1][idx_dict[1]:]
                    supercontig = before_bk_string + after_bk_string
                    if contig_num_dict[0] <= max_contig_num:
                        use_for_supercontig_dict[contig_num_dict[0]] = 1
                    if contig_num_dict[1] <= max_contig_num:
                        use_for_supercontig_dict[contig_num_dict[1]] = 1
                    del contig_info_dict[contig_num_dict[0]]
                    del contig_info_dict[contig_num_dict[1]]
                    new_contig_num += 1
                    PS_str = contig_PS_dict[0] + "-" + contig_PS_dict[1]
                    contig_info_dict[new_contig_num] = [new_contig_num,chr_num,supercontig,PS_str]
                    #print(len(contig_seq_dict[0]),len(contig_seq_dict[1]),len(supercontig))
                elif before_bk_len[0] >= before_bk_len[1] and after_bk_len[0] >= after_bk_len[1]:
                    if contig_num_dict[1] <= max_contig_num:
                        remove_contig_dict[contig_num_dict[1]] = 1
                elif before_bk_len[1] >= before_bk_len[0] and after_bk_len[1] >= after_bk_len[0]:
                    if contig_num_dict[0] <= max_contig_num:
                        remove_contig_dict[contig_num_dict[0]] = 1
                else:
                    print("wrong")
                    count_no_supercontig += 1

            elif len(contig_merge) == 1:
                count_1 += 1
            elif len(contig_merge) > 2:
                count_3 += 1

        print("finishe one big phase block")
        for _contig_num,_contig_one in contig_info_dict.items():
            if _contig_num > max_contig_num:
                supercontig_dict[count_supercontig,key] = _contig_one
                count_supercontig += 1

    print("finished~")
    return (supercontig_dict,use_for_supercontig_dict,remove_contig_dict)


def Finalize_contig_fasta_file(contig_fasta_file,supercontig_dict,use_for_supercontig_dict,remove_contig_dict,output_file):
    fw = open(output_file,"w")
    f = open(contig_fasta_file,"r")
    count = 1
    remove_flag = 0
    count =  1
    count_contig =  1
    curr = 0
    for line in f:
        curr += 1
        data = line.rsplit()
        if count%2 == 1:
            contig_num = int(data[0].split("_")[0][1:])
            PS_flag_1 = int(data[0].split("_")[1].split("PS")[1].split(":")[0])
            PS_flag_2 = int(data[0].split("_")[1].split("PS")[1].split(":")[1])
            HP_flag = data[0].split("_")[2]
            if contig_num in use_for_supercontig_dict or contig_num in remove_contig_dict:
                remove_flag = 1
        elif count%2 == 0:
            if remove_flag == 1:
                remove_flag = 0
            else:
                fw.writelines(">" + str(count_contig) + "_PS" + str(PS_flag_1) + ":" + str(PS_flag_2) + "_" + HP_flag +  "\n")
                fw.writelines(line)
                count_contig += 1
                remove_flag = 0
        count += 1
    
    for key,val in supercontig_dict.items():
        contig_seq = val[2]
        PS_flag_1 = key[1][0]
        PS_flag_2 = key[1][1]
        HP_flag = key[1][2]
        stitch_info = val[-1]
        fw.writelines(">" + str(count_contig) + "_PS" + str(PS_flag_1) + ":" + str(PS_flag_2) + "_" + HP_flag + "_merge" + stitch_info + "\n")
        fw.writelines(contig_seq + "\n")
        count_contig += 1
 
    f.close()
    fw.close()

    print("done")


def Contig_start(chr_num,cut_threshold,ref_dir,out_dir,phase_cut_folder,xin):
    HCbk_file = phase_cut_folder + "chr" + str(chr_num) +  ".phased_final_cut_by_" + str(cut_threshold) + "_HC_breakpoint_2.p"
    contig_fasta_file = out_dir + "Aquila_cutPBHC_minicontig_chr" + str(chr_num) + ".fasta"
    contig_sam_file = out_dir + "Aquila_cutPBHC_minicontig_chr" + str(chr_num) + ".sam"
    ref_fasta_file = ref_dir + "ref_seq_chr" + str(chr_num) + ".p"
    ref_fasta_dict = pickle.load(open(ref_fasta_file,"rb"))
    output_file = out_dir + "Aquila_Contig_chr" + str(chr_num) + ".fasta"

    supercontig_dict,use_for_supercontig_dict,remove_contig_dict = Make_supercontig_based_on_HCbk(HCbk_file,contig_sam_file,ref_fasta_dict,"chr" + str(chr_num))
    Finalize_contig_fasta_file(contig_fasta_file,supercontig_dict,use_for_supercontig_dict,remove_contig_dict,output_file)




import pdb
#pdb.set_trace()
from collections import defaultdict
import pickle
import math
import os
import numpy as np
from Assign_phase_block_v3 import * 
import sys
import operator
from scipy.special import comb
import glob
from subprocess import Popen



def split_h5(h5_file,chr_num,output_dir):
    f = open(h5_file,"r")
    file_num = 1
    filename = output_dir + "chr" + str(chr_num) + "_" +  str(file_num)
    fw = open(filename,"w")
    mole_end_max = 0
    count = 0
    curr = 0
    for line in f:
        #print(curr)
        curr += 1
        data = line.rsplit()
        if len(data) >= 9:
            mole_start = int(data[1])
            mole_end = int(data[2])
            if mole_start > mole_end_max and count > 0:
                fw.close()
                if count > 0 and count <= 10000 and file_num > 1:
                    prev_filename = output_dir + "chr" + str(chr_num) + "_" +  str(file_num-1)
                    temp_file = "chr_of_" + str(chr_num) + "_temp"
                    Popen('cat ' + prev_filename + ' ' + filename + ' > ' + output_dir  +  temp_file,shell=True).wait()
                    Popen('mv ' + output_dir + temp_file + ' ' + prev_filename,shell=True).wait()
                    count = 0
                    mole_end_max = mole_end
                    filename = output_dir + "chr" + str(chr_num) + "_" +  str(file_num)
                    fw = open(filename,"w")
                    fw.writelines(line)
                    count += 1

                else:
                    count = 0
                    mole_end_max = mole_end
                    file_num += 1
                    filename = output_dir + "chr" + str(chr_num) + "_" +  str(file_num)
                    fw = open(filename,"w")
                    fw.writelines(line)
                    count += 1

            else:
                if mole_end > mole_end_max:
                    mole_end_max = mole_end
                
                fw.writelines(line)
                count += 1

    print("Raw clustering finished...")
    #print(file_num)
    return file_num


def Get_h5_for_mole_with_one_hetero_var(h5_file,chr_num,output_dir):
    f = open(h5_file,"r")
    filename = output_dir + "chr" + str(chr_num) + "_one_var"
    fw = open(filename,"w")
    for line in f:
        data = line.rsplit()
        if len(data) == 8:
            fw.writelines(line)

    print("Extract molecule with one heterozygous variant finished...")


def Generate_hyplotype_for_one_cluster(merge_var,merge_var_complement):
    use_var_dict = defaultdict(int)
    one_cluster_var_dict = defaultdict(int)
    count_all = 0
    count_conflict = 0
    _times = 3
    for var_pos, all_hp in merge_var.items():
        all_hp_complement = merge_var_complement[var_pos]
        if all_hp_complement == []:
            if len(set(all_hp)) == 1 and len(all_hp) >= 2:
                one_cluster_var_dict[var_pos] = all_hp[0] + 1
                count_all += 1
            else:
                if len(all_hp) >= 2:
                    count1 = all_hp.count(1)
                    count0 = all_hp.count(0)
                    if count1 >= _times*count0:
                        one_cluster_var_dict[var_pos] = 2 # 1
                    elif count0 >= _times*count1:
                        one_cluster_var_dict[var_pos] = 1 # 0
                    else:
                        count_conflict += 1
                    count_all += 1
                else:
                    count_conflict += 1
        else:
            count_all += 1
            use_var_dict[var_pos] = 1
            count1 = all_hp.count(1)
            count0 = all_hp.count(0)
            if count1 > _times*count0:
                use_hp = 2
            elif count0 > _times*count1:
                use_hp = 1
            else:
                use_hp = "nan"

            count1_complement = all_hp_complement.count(1)
            count0_complement = all_hp_complement.count(0)
            if count1_complement >= _times*count0_complement:
                use_hp_complement = 2
            elif count0_complement >= _times*count1_complement:
                use_hp_complement = 1
            else:
                use_hp_complement = "nan"
            if use_hp == "nan" and use_hp_complement == "nan":
                count_conflict += 1
            else:
                if use_hp == use_hp_complement:
                    count_conflict += 1
                else:
                    if use_hp == "nan":
                        if count1 > count0 and use_hp_complement == 1:
                            one_cluster_var_dict[var_pos] = get_complement_hp(use_hp_complement)
                        elif count0 > count1 and use_hp_complement == 2:
                            one_cluster_var_dict[var_pos] = get_complement_hp(use_hp_complement)
                        else:
                            count_conflict += 1
                    elif use_hp_complement == "nan":
                        if count1_complement > count0_complement and use_hp == 1:
                            one_cluster_var_dict[var_pos] = use_hp
                        elif count0_complement > count1_complement and use_hp == 2:
                            one_cluster_var_dict[var_pos] = use_hp
                        else:
                            count_conflict += 1
                    else:
                        one_cluster_var_dict[var_pos] = use_hp

    # process the uniq variant from complement molecule
    for var_pos, all_hp in merge_var_complement.items():
        if use_var_dict[var_pos] != 1 and all_hp != []:
            if len(set(all_hp)) == 1 and len(all_hp) >= 2 :
                one_cluster_var_dict[var_pos] = get_complement_hp_2(all_hp[0]) + 1
                count_all += 1
            else:
                if len(all_hp) >= 2:
                    count1 = all_hp.count(1)
                    count0 = all_hp.count(0)
                    if count1 >= _times*count0:
                        one_cluster_var_dict[var_pos] = 1 # 0
                    elif count0 >= _times*count1:
                        one_cluster_var_dict[var_pos] = 2 # 1
                    else:
                        count_conflict += 1
                    count_all += 1
                else:
                    count_conflict += 1

    conflict_percent = float(count_conflict)/count_all
    return (one_cluster_var_dict,conflict_percent)


def Generate_hyplotype_for_all_cluster(mole_dict,all_merge_cluster_dict, all_merge_cluster_complement_dict):
    final_cluster = []
    count_total = 0
    conflict_list = []
    for num, one_cluster, in all_merge_cluster_dict.items():
        merge_var = defaultdict(list)
        merge_var_complement = defaultdict(list)
        one_cluster_complement = all_merge_cluster_complement_dict[num]
        count_total += len(one_cluster)
        for mole_num in one_cluster:
            mole = mole_dict[mole_num]
            for var_pos,var_hp in mole.items():
                merge_var[var_pos].append(var_hp)
        for mole_num in one_cluster_complement:
            mole = mole_dict[mole_num]
            for var_pos,var_hp in mole.items():
                merge_var_complement[var_pos].append(var_hp)
        
        #print("------------")
        #print(merge_var)
        #print(merge_var_complement)
        #print("------------")

        one_cluster_var_dict,conflict_percent = Generate_hyplotype_for_one_cluster(merge_var,merge_var_complement)
        conflict_list.append(conflict_percent)
        final_cluster.append(one_cluster_var_dict)
        merge_var = defaultdict(list)

    #print(count_total)
    #print(len(all_merge_cluster_dict))
    #print("average conflict percent: " + str(np.mean(conflict_list)))
    return final_cluster

 
def get_complement_hp(hp):
    if hp == 1:
        return 2
    elif hp == 2:
        return 1


def get_complement_hp_2(hp):
    if hp == 1:
        return 0
    elif hp == 0:
        return 1


def nCr(n,r):
    f = math.factorial
    return float(f(n)) / (f(r) * f(n-r))


def Recursive_Clustering_for_Overlap_Variants(final_cluster,step,h5_filename,overlap_var_threshold):
    cluster_list = []
    curr = 0
    count_nonmerge = 0
    for idx in range(len(final_cluster)):
        var_dict = final_cluster[idx]
        #print(curr)
        curr += 1
        merge_cluster = 0
        if cluster_list != []:
            for num in range(len(cluster_list)):
                one_cluster = cluster_list[num]
                overlap_num = 0
                break_flag = 0
                for locus,hp in var_dict.items():
                    for locus_2,hp_2 in one_cluster.items():
                        if locus == locus_2:
                            overlap_num += 1
                            if overlap_num >= overlap_var_threshold:
                                break_flag = 1
                                break
                    if break_flag == 1:
                        break

                if overlap_num >= overlap_var_threshold:
                    uniq_var = []
                    conflict_var = []
                    nonconflict_var = []
                    for locus,hp in var_dict.items():
                        var_hp_1 = one_cluster[locus]
                        var_hp_2 = var_dict[locus]
                        if (var_hp_1 == 0 and var_hp_2 == 1 ) or (var_hp_1 == 0 and var_hp_2 == 2):
                            uniq_var.append(locus)
                        elif (var_hp_1 == 1 and var_hp_2 == 2) or (var_hp_1 == 2 and var_hp_2 == 1):
                            conflict_var.append(locus)
                        elif (var_hp_1 == 1 and var_hp_2 == 1) or (var_hp_1 == 2 and var_hp_2 == 2):
                            nonconflict_var.append(locus)

                    if len(conflict_var) >= len(nonconflict_var)*5.0:   # default = 3
                        for locus in nonconflict_var:
                            one_cluster.pop(locus)
                        for locus in uniq_var:
                            one_cluster[locus] = get_complement_hp(var_dict[locus])

                    elif len(nonconflict_var) >= len(conflict_var)*5.0:
                        for locus in conflict_var:
                            one_cluster.pop(locus)
                        for locus in uniq_var:
                            one_cluster[locus] = var_dict[locus]

                    else:
                        count_nonmerge += 1
                        merge_cluster = 0
                        continue

                    merge_cluster = 1
                    cluster_list[num] = one_cluster     # update, add 1-15-2018
                    break

        if merge_cluster == 0:
            cluster_list.append(var_dict)


    prev_cluster_num = len(final_cluster)

    if len(cluster_list) == prev_cluster_num:
        print("xin here: ")
        print("Clustering " +  str(step) + ":  " + str(count_nonmerge))
        print(prev_cluster_num,len(cluster_list), prev_cluster_num - len(cluster_list))
        print("Converged...")
        save_file = h5_filename + ".p"
        pickle.dump(cluster_list,open(save_file, "wb"))

        return 0

    step += 1
    if step == 1:
        print("Clustering " +  str(step)+ ":  " + str(count_nonmerge) + "," + str(round(float(count_nonmerge/prev_cluster_num),3)) )
    else:
        print("Clustering " +  str(step)+ ":  " + str(count_nonmerge)  )
    #print(prev_cluster_num,len(cluster_list), prev_cluster_num - len(cluster_list))
    final_cluster = cluster_list
    Recursive_Clustering_for_Overlap_Variants(final_cluster,step,h5_filename,overlap_var_threshold)
                    

def Clustering_for_phaseblock(mole_dict,all_merge_cluster_dict, all_merge_cluster_complement_dict,h5_filename,overlap_var_threshold):
    final_cluster = Generate_hyplotype_for_all_cluster(mole_dict,all_merge_cluster_dict, all_merge_cluster_complement_dict)
    Recursive_Clustering_for_Overlap_Variants(final_cluster,0,h5_filename,overlap_var_threshold)


def Recursive_clustering(all_merge_cluster_dict,all_merge_cluster_complement_dict,all_merge_cluster_dict_start_end, all_merge_cluster_complement_dict_start_end,step,mole_dict,mole_info,h5_filename,overlap_var_threshold,support_threshold):
    support_threshold = int(support_threshold)
    all_merge_cluster_dict_sorted = Sort_All_Clusters_by_Start(all_merge_cluster_dict,all_merge_cluster_dict_start_end)
    all_merge_cluster_dict = {}
    all_merge_cluster_dict = all_merge_cluster_dict_sorted
    cluster_dict = defaultdict(list)
    cluster_complement_dict = defaultdict(list)
    cluster_dict_start_end = defaultdict(list)
    cluster_complement_dict_start_end = defaultdict(list)
    curr = 0
    cluster_num = 0
    total_num = len(all_merge_cluster_dict)
    for _num,mole_num_list in all_merge_cluster_dict.items():
       # print(curr)
        curr += 1
        merge_cluster_flag = 0
        cluster = mole_num_list
        cluster_start = all_merge_cluster_dict_start_end[_num][0]
        cluster_end = all_merge_cluster_dict_start_end[_num][1]

        cluster_complement = all_merge_cluster_complement_dict[_num] 
        cluster_complement_start = all_merge_cluster_complement_dict_start_end[_num][0]
        cluster_complement_end = all_merge_cluster_complement_dict_start_end[_num][1]
        
        if cluster_dict != {}:
            for num, cluster_comp in cluster_dict.items():
                cluster_comp_start = cluster_dict_start_end[num][0]
                cluster_comp_end = cluster_dict_start_end[num][1]
                cluster_comp_complement = cluster_complement_dict[num] 
 
                if cluster_start <= cluster_comp_end:
                    overlap_num = 0
                    for mole_i in cluster:
                        for mole_j in cluster_comp:
                            if mole_i == mole_j:
                                overlap_num += 1
                                if overlap_num >= support_threshold:
                                    merge_cluster_flag = 1
                                    cluster_dict[num] = cluster.union(cluster_comp)
                                    cluster_complement_dict[num] = cluster_complement.union(cluster_comp_complement)
                                    cluster_new_start, cluster_new_end = Get_start_end_for_cluster(cluster_dict[num],mole_info)
                                    cluster_complement_new_start, cluster_complement_new_end = Get_start_end_for_cluster(cluster_complement_dict[num],mole_info)
                                    cluster_dict_start_end[num] = [cluster_new_start, cluster_new_end]
                                    cluster_complement_dict_start_end[num] = [cluster_complement_new_start, cluster_complement_new_end]

                                    break
                        if merge_cluster_flag == 1:
                            break
                    if merge_cluster_flag == 1:
                        break
            if merge_cluster_flag == 0:
                for num, cluster_comp in cluster_complement_dict.items():
                    cluster_comp_start = cluster_complement_dict_start_end[num][0]
                    cluster_comp_end = cluster_complement_dict_start_end[num][1]
                    cluster_comp_complement = cluster_dict[num] 
     
                    if cluster_start <= cluster_comp_end:
                        overlap_num = 0
                        for mole_i in cluster:
                            for mole_j in cluster_comp:
                                if mole_i == mole_j:
                                    overlap_num += 1
                                    if overlap_num >= support_threshold:
                                        merge_cluster_flag = 1
                                        cluster_complement_dict[num] = cluster.union(cluster_comp)
                                        cluster_dict[num] = cluster_complement.union(cluster_comp_complement)
                                        cluster_new_start, cluster_new_end = Get_start_end_for_cluster(cluster_complement_dict[num],mole_info)
                                        cluster_complement_new_start, cluster_complement_new_end = Get_start_end_for_cluster(cluster_dict[num],mole_info)
                                        cluster_complement_dict_start_end[num] = [cluster_new_start, cluster_new_end]
                                        cluster_dict_start_end[num] = [cluster_complement_new_start, cluster_complement_new_end]

                                        break
                            if merge_cluster_flag == 1:
                                break
                        if merge_cluster_flag == 1:
                            break
        
        if merge_cluster_flag == 0:
            cluster_dict[cluster_num] = cluster
            cluster_complement_dict[cluster_num] = cluster_complement
            cluster_dict_start_end[cluster_num] = [cluster_start, cluster_end]
            cluster_complement_dict_start_end[cluster_num] = [cluster_complement_start, cluster_complement_end]
            cluster_num += 1
   
    prev_num = total_num
    all_merge_cluster_dict = cluster_dict    
    all_merge_cluster_dict_start_end = cluster_dict_start_end 
    all_merge_cluster_complement_dict = cluster_complement_dict    
    all_merge_cluster_complement_dict_start_end = cluster_complement_dict_start_end   

    step += 1
    print("Clustering " + str(step) + ":")
    print(total_num,len(cluster_dict),len(cluster_complement_dict), total_num - len(cluster_dict))
    if prev_num == len(all_merge_cluster_dict):
        print("Converged...")
        print(len(all_merge_cluster_dict))
        Clustering_for_phaseblock(mole_dict,all_merge_cluster_dict, all_merge_cluster_complement_dict,h5_filename,overlap_var_threshold)

        return 1

    Recursive_clustering(all_merge_cluster_dict,all_merge_cluster_complement_dict,all_merge_cluster_dict_start_end, all_merge_cluster_complement_dict_start_end,step,mole_dict,mole_info,h5_filename,overlap_var_threshold,int(support_threshold))


def Get_start_end_for_cluster(one_cluster_mole_list,mole_info):
    count = 1
    for mole_num in one_cluster_mole_list:
        mole_start_and_end = mole_info[mole_num]
        mole_start = mole_start_and_end[0]
        mole_end = mole_start_and_end[1]
        if count == 1:
            start_min = mole_start
            end_max = mole_end
        else:
            if start_min > mole_start:
                start_min = mole_start 
            if end_max < mole_end:
                end_max = mole_end

        count += 1

    return (start_min, end_max)


def Sort_All_Clusters_by_Start(all_merge_cluster_dict,all_merge_cluster_dict_start_end):
    all_merge_cluster_dict_start = defaultdict(int)
    for key,val in all_merge_cluster_dict_start_end.items():
        all_merge_cluster_dict_start[key] = val[0]

    all_merge_cluster_dict_start_sorted = sorted(all_merge_cluster_dict_start.items(), key=operator.itemgetter(1))
    sorted_keys = []
    total_num = len(all_merge_cluster_dict_start_sorted)
    for ii in range(total_num):
        key_value = all_merge_cluster_dict_start_sorted[ii]
        key = key_value[0]
        sorted_keys.append(key)

    all_merge_cluster_dict_sorted = defaultdict(list)
    for key in sorted_keys:
        all_merge_cluster_dict_sorted[key] = all_merge_cluster_dict[key]

    return all_merge_cluster_dict_sorted


def process_chr(h5_file,overlap_var_threshold, support_threshold,var_depth_used,xin):
    support_threshold = int(support_threshold)
    f = open(h5_file,"r")
    curr = 0
    merge_dict = defaultdict(list)
    mole_dict = defaultdict()
    mole_info = defaultdict(list)
    for line in f:
        #print(curr)
        curr += 1
        data = line.rsplit()
        #chr_num = int(data[0])
        mole_num = int(data[6])
        mole_start = int(data[1])
        mole_end = int(data[2])
        var_list = data[7:]
        var_dict = defaultdict(int)
        var_hetero_pos = []
        var_hetero_hp = defaultdict(int)
        for var in var_list:
            var_info = var.split(":")
            var_pos = int(var_info[0])
            var_hp = int(var_info[1])
            var_dict[var_pos] = var_hp
            var_hetero_pos.append(var_pos)
            var_hetero_hp[var_pos] = var_hp
        var_hetero_pos_sorted = sorted(var_hetero_pos)

        for i in range(len(var_hetero_pos_sorted)-1):
            pos_1 = var_hetero_pos_sorted[i]
            pos_2 = var_hetero_pos_sorted[i+1]
            hp_1 = var_hetero_hp[pos_1]
            hp_2 = var_hetero_hp[pos_2]
            merge_dict[(pos_1,pos_2,hp_1,hp_2)].append(mole_num)

        """
        for i in range(len(var_hetero_pos_sorted)-1):
            for j in range(i + 1, len(var_hetero_pos_sorted)):
                pos_1 = var_hetero_pos_sorted[i]
                pos_2 = var_hetero_pos_sorted[j]
                hp_1 = var_hetero_hp[pos_1]
                hp_2 = var_hetero_hp[pos_2]
                merge_dict[(pos_1,pos_2,hp_1,hp_2)].append(mole_num)
        """
        mole_dict[mole_num] = var_dict
        mole_info[mole_num] = [mole_start,mole_end]

    count_merge = 0
    count_nomerge = 0
    all_merge_cluster_dict = defaultdict(list)
    all_merge_cluster_dict_start_end = defaultdict(list)
    for key,val in merge_dict.items():
        var_1_pos = key[0]
        var_2_pos = key[1]
        var_1_hp = key[2]
        var_2_hp = key[3]

        val_len = len(val)
        if (var_1_pos,var_2_pos,get_complement_hp_2(var_1_hp),get_complement_hp_2(var_2_hp)) in merge_dict:
            val2_len = len(merge_dict[(var_1_pos,var_2_pos,get_complement_hp_2(var_1_hp),get_complement_hp_2(var_2_hp))]) 
        else:
            val2_len = 0
            
        if (var_1_pos,var_2_pos,get_complement_hp_2(var_1_hp),var_2_hp) in merge_dict:
            val3_len = len(merge_dict[(var_1_pos,var_2_pos,get_complement_hp_2(var_1_hp),var_2_hp)]) 
        else:
            val3_len = 0
            
        if (var_1_pos,var_2_pos,var_1_hp,get_complement_hp_2(var_2_hp)) in merge_dict:
            val4_len = len(merge_dict[(var_1_pos,var_2_pos,var_1_hp,get_complement_hp_2(var_2_hp))]) 
        else:
            val4_len = 0
        
        p1 = 0.95
        p2 = 0.95
        n_total = val_len + val2_len + val3_len + val4_len
        k_false = val3_len + val4_len
        k_true = val_len + val2_len
        n_choose_k_false = int(comb(n_total, k_false, exact=True))
        total_combinations = 0
        for kk in range(n_total):
            n_choose_kk = int(comb(n_total, kk + 1, exact=True))
            total_combinations += n_choose_kk
        try:
            ratio = float(n_choose_k_false)/total_combinations
            final_prob = n_choose_k_false*math.pow((1-p1*p2),k_false)*math.pow((p1*p2),k_true)
        except:
            #print(val_len,val2_len,val3_len,val4_len)
            ratio = 1.0
            final_prob = 0.0

        if float(final_prob)/ratio > 0.9999 and (val_len >= var_depth_used and val2_len >= var_depth_used):
            count_merge += 1
            all_merge_cluster_dict[key] = val
            cluster_start_min, cluster_end_max = Get_start_end_for_cluster(val,mole_info)
            all_merge_cluster_dict_start_end[key] = [cluster_start_min, cluster_end_max]
            #print(val_len,val2_len,val3_len,val4_len)

        else:
            count_nomerge += 1
            #print(val_len,val2_len,val3_len,val4_len)



    #print(count_merge,count_nomerge)

    # sort all the clusters by start
    all_merge_cluster_dict_sorted = Sort_All_Clusters_by_Start(all_merge_cluster_dict,all_merge_cluster_dict_start_end)
    all_merge_cluster_dict = {}
    all_merge_cluster_dict = all_merge_cluster_dict_sorted
    # start first clustering
    cluster_dict = defaultdict(list)
    cluster_complement_dict = defaultdict(list)
    cluster_dict_start_end = defaultdict(list)
    cluster_complement_dict_start_end = defaultdict(list)
    curr = 0
    cluster_num = 0
    use_cluster_dict = defaultdict(int)
    total_num = len(all_merge_cluster_dict)
    
    for key,mole_num_list in all_merge_cluster_dict.items():
        if use_cluster_dict[key] != 1:
           # print(curr)
            curr += 1
            merge_cluster_flag = 0
            cluster = set(mole_num_list)
            cluster_start = all_merge_cluster_dict_start_end[key][0]
            cluster_end = all_merge_cluster_dict_start_end[key][1]
            var_1_pos = key[0]
            var_2_pos = key[1]
            var_1_hp = key[2]
            var_2_hp = key[3]
            key_complement = (var_1_pos,var_2_pos,get_complement_hp_2(var_1_hp),get_complement_hp_2(var_2_hp))  
            #if key_complement not in all_merge_cluster_dict:   # add
                #continue     # add
            cluster_complement = set(all_merge_cluster_dict[key_complement]) 
            cluster_complement_start = all_merge_cluster_dict_start_end[key_complement][0]
            cluster_complement_end = all_merge_cluster_dict_start_end[key_complement][1]
            use_cluster_dict[key_complement] = 1
            
            if cluster_dict != {}:
                for num, cluster_comp in cluster_dict.items():
                    cluster_comp_start = cluster_dict_start_end[num][0]
                    cluster_comp_end = cluster_dict_start_end[num][1]
                    cluster_comp_complement = cluster_complement_dict[num] 
     
                    if cluster_start <= cluster_comp_end:
                        overlap_num = 0
                        for mole_i in cluster:
                            for mole_j in cluster_comp:
                                if mole_i == mole_j:
                                    overlap_num += 1
                                    if overlap_num >= support_threshold:
                                        merge_cluster_flag = 1
                                        cluster_dict[num] = cluster.union(cluster_comp)
                                        cluster_complement_dict[num] = cluster_complement.union(cluster_comp_complement)
                                        cluster_new_start, cluster_new_end = Get_start_end_for_cluster(cluster_dict[num],mole_info)
                                        cluster_complement_new_start, cluster_complement_new_end = Get_start_end_for_cluster(cluster_complement_dict[num],mole_info)
                                        cluster_dict_start_end[num] = [cluster_new_start, cluster_new_end]
                                        cluster_complement_dict_start_end[num] = [cluster_complement_new_start, cluster_complement_new_end]

                                        break
                            if merge_cluster_flag == 1:
                                break
                        if merge_cluster_flag == 1:
                            break
                if merge_cluster_flag == 0:
                    for num, cluster_comp in cluster_complement_dict.items():
                        cluster_comp_start = cluster_complement_dict_start_end[num][0]
                        cluster_comp_end = cluster_complement_dict_start_end[num][1]
                        cluster_comp_complement = cluster_dict[num] 
         
                        if cluster_start <= cluster_comp_end:
                            overlap_num = 0
                            for mole_i in cluster:
                                for mole_j in cluster_comp:
                                    if mole_i == mole_j:
                                        overlap_num += 1
                                        if overlap_num >= support_threshold:
                                            merge_cluster_flag = 1
                                            cluster_complement_dict[num] = cluster.union(cluster_comp)
                                            cluster_dict[num] = cluster_complement.union(cluster_comp_complement)
                                            cluster_new_start, cluster_new_end = Get_start_end_for_cluster(cluster_complement_dict[num],mole_info)
                                            cluster_complement_new_start, cluster_complement_new_end = Get_start_end_for_cluster(cluster_dict[num],mole_info)
                                            cluster_complement_dict_start_end[num] = [cluster_new_start, cluster_new_end]
                                            cluster_dict_start_end[num] = [cluster_complement_new_start, cluster_complement_new_end]

                                            break
                                if merge_cluster_flag == 1:
                                    break
                            if merge_cluster_flag == 1:
                                break


            
            if merge_cluster_flag == 0:
                cluster_dict[cluster_num] = cluster
                cluster_complement_dict[cluster_num] = cluster_complement
                cluster_dict_start_end[cluster_num] = [cluster_start, cluster_end]
                cluster_complement_dict_start_end[cluster_num] = [cluster_complement_start, cluster_complement_end]
                cluster_num += 1
            
    print("Clustering " + str(1) + ":")
    #print(total_num,len(cluster_dict),len(cluster_complement_dict), total_num - len(cluster_dict))

    all_merge_cluster_dict = cluster_dict    
    all_merge_cluster_dict_start_end = cluster_dict_start_end 
    all_merge_cluster_complement_dict = cluster_complement_dict    
    all_merge_cluster_complement_dict_start_end = cluster_complement_dict_start_end   
    Recursive_clustering(all_merge_cluster_dict,all_merge_cluster_complement_dict,all_merge_cluster_dict_start_end, all_merge_cluster_complement_dict_start_end,1,mole_dict,mole_info,h5_file,overlap_var_threshold,support_threshold)


def Phase_start(output_dir,h5_dir,sample_name,chr_start,chr_end,overlap_var_threshold,support_threshold,xin):
    """ Algorithm to assign RAW phase blocks"""
    for chr_num in range(chr_start,chr_end + 1):
        rm_file = output_dir + "chr" + str(chr_num) + "_*"
        del_file_list = glob.glob(rm_file)
        if del_file_list != []:
            Popen("rm " + rm_file,shell=True).wait()
        rm_file = output_dir + "chr" + str(chr_num) + ".phased_final*"
        del_file_list = glob.glob(rm_file)
        print(del_file_list)
        if del_file_list != []:
            Popen("rm " + rm_file,shell=True).wait()

    print("using overlap variants threshold: " + str(overlap_var_threshold)) 
    print("using molecules support threshold: " + str(support_threshold)) 
    print("here: " + output_dir)
    #hetero_var_dict = pickle.load(open(h5_dir + "variant_dict_heterozygous.p","rb"))
    f = open(h5_dir + "median_depth_for_var.txt","r")
    for line in f :
        data = line.rsplit()
        var_depth_median = float(data[0])
    f.close()
    var_depth_used = var_depth_median*0.2
    for chr_num in range(chr_start,chr_end+1):
        print("processing " + str(chr_num) + "...")
        mole_h5_file_origin = h5_dir + sample_name + "_chr" + str(chr_num) + "_sorted.h5"
        total_filenum = split_h5(mole_h5_file_origin,chr_num,output_dir)
        for i in range(total_filenum):
            cur_filenum = i + 1
            cur_filename = output_dir + "chr" + str(chr_num) + "_" + str(cur_filenum)
            #print(cur_filename)
            process_chr(cur_filename,overlap_var_threshold,support_threshold,var_depth_used,"xin")

        """ Assign FINAL phase blocks """
        metric_phase_percent = []
        metric_corr_percent = []
        for i in range(total_filenum):
            cur_filenum = i + 1
            cur_filename = output_dir + "chr" + str(chr_num) + "_" + str(cur_filenum)
            phase_block_file = cur_filename + ".p"
            output_file_raw = cur_filename + ".phased.raw"
            output_file = cur_filename + ".phased"
            print(cur_filename)
            read_phase_block_file(phase_block_file,cur_filename,output_file_raw,metric_phase_percent, metric_corr_percent)
            Finalize_phase_block(output_file_raw, output_file)

        print("------Final Results for chr" + str(chr_num) + "-------")
        #print("phased percent: " + str(np.mean(metric_phase_percent)) + ", " +str(np.median(metric_phase_percent)))
        #print("correct percent: " + str(np.mean(metric_corr_percent)) + ", " +str(np.median(metric_corr_percent)))

        for file_ in glob.glob(output_dir + "chr" + str(chr_num) + "_*.phased"):
            Popen("cat " + file_ + ">> " +  output_dir + "chr" + str(chr_num) + ".phased_final_1",shell=True).wait()

        

        """ Assign phase block for molecule with only one heterzygous variant"""
        phase_file_final_1 = output_dir + "chr" + str(chr_num) + ".phased_final_1"

        Get_h5_for_mole_with_one_hetero_var(mole_h5_file_origin,chr_num,output_dir)

        h5_file_for_mole_with_one_var = output_dir + "chr" + str(chr_num) + "_one_var"
        phase_file_for_mole_with_one_var = h5_file_for_mole_with_one_var + ".phased"
        nonphase_dict = defaultdict(list) 

        var_prob_dict = Calculate_GenotypeProb_for_variants(phase_file_final_1)
        nonphase_dict = Assign_phase_block_for_mole_with_one_var(var_prob_dict,h5_file_for_mole_with_one_var,phase_file_for_mole_with_one_var,nonphase_dict)



        """ Impute the phase block for the rest very few molecules """
        phase_file_final_2 = output_dir +  "chr" + str(chr_num) + ".phased_final_2"

        nonphase_dict = Impute_phase_block(var_prob_dict,phase_file_final_1,phase_file_final_2,nonphase_dict)
        

        h5_file_for_mole_with_one_var = output_dir + "chr" + str(chr_num) + "_one_var"

        phase_file_for_mole_with_one_var_2 = h5_file_for_mole_with_one_var + ".phased_2"
        phase_file_final_3 = output_dir +  "chr" + str(chr_num) + ".phased_final_3"

        Impute_nonphase_variant(nonphase_dict,phase_file_for_mole_with_one_var,phase_file_final_2,phase_file_for_mole_with_one_var_2,phase_file_final_3)

        
        phase_file_final_total = output_dir +  "chr" + str(chr_num) + ".phased_final"
        #### cancatenate all phased files together ####
        Popen("cat " + phase_file_for_mole_with_one_var_2 + " " + phase_file_final_3 + " > " + phase_file_final_total,shell=True).wait()




import pdb
#pdb.set_trace()
from collections import defaultdict
import pickle
import numpy as np
from argparse import ArgumentParser
import os
import sys
parser = ArgumentParser(description="Variants Calling:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=22)
parser.add_argument('--var_size','-v',type=int,help="variant size", default=1)
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs", default='results_/')

args = parser.parse_args()

def Extract_SV_info(hap_SV_file,output_file,chr_num,v_size):
    f = open(hap_SV_file,"r")
    contig_dict = defaultdict(list)
    curr = 0
    for line in f:
        #print(curr)
        curr += 1
        data = line.rsplit()
        if data[0] == "V":
            #print(data)
            mapq = int(data[5])
            repeat = int(data[4])
            cur_chr_num = data[1]
            if cur_chr_num == "chrX":
                cur_chr_num = "chr23"
            if mapq >= 20 and repeat == 1 and cur_chr_num == "chr" + str(chr_num): 
                cur_chr_num = data[1]
                ref_start = int(data[2])
                ref_end = int(data[3])
                ref = data[6]
                alt = data[7]
                contig_num = data[8]
                contig_start = int(data[9])
                contig_end = int(data[10])
                strand_dir = data[-1]
                contig_dict[(cur_chr_num,ref_start,ref_end)].append([cur_chr_num,ref_start, ref_end, ref, alt, contig_num,contig_start,contig_end,strand_dir])

    contig_dict_filter = defaultdict(list)
    count = 0
    for key, val in contig_dict.items():
        if len(val) > 1:
            #print(">1:")
            #print(val)
            count += 1
        else:
            contig_dict_filter[key] = val[0]

    SV_dict = defaultdict(list)
    for key, val in contig_dict_filter.items():
        ref = val[3] 
        alt = val[4]
        len_var = abs(len(ref)-len(alt))
        #if len_var >= 50 or (alt == "-" and len_var == 49) or (ref == "-" and len_var == 49):
        #if len_var >= 1 or (alt == "-" and len_var == 0) or (ref == "-" and len_var == 0):
        #if len_var >= v_size or (alt == "-" and len_var == v_size - 1) or (ref == "-" and len_var == v_size - 1):
        if (ref == "-" and len(alt) >= v_size) or (alt == "-" and len(ref) >= v_size):
            SV_dict[key] = val
        
    pickle.dump(SV_dict, open(output_file,"wb"))

    return SV_dict


def check_start_end(chr_start_list,chr_end_list):
    chr_start_used = np.min(chr_start_list)
    chr_end_used = np.max(chr_end_list)
    return (chr_start_used,chr_end_used)


def compare_two_haploid_SV(SV_dict_contig_1_file, SV_dict_contig_2_file,chr_num,ref_dict_contig_1_file,ref_dict_contig_2_file,out_dir):
    del_homo_dict = defaultdict(int)
    del_hetero_dict = defaultdict(int)
    del_homo_sv = defaultdict(list)
    del_hetero_sv = defaultdict(list)
    SV_dict_contig_1 = pickle.load(open(SV_dict_contig_1_file,"rb"))
    SV_dict_contig_2 = pickle.load(open(SV_dict_contig_2_file,"rb"))
    ref_dict_contig_1 = pickle.load(open(ref_dict_contig_1_file,"rb"))
    ref_dict_contig_2 = pickle.load(open(ref_dict_contig_2_file,"rb"))
    count_total_share = 0
    homo_key_dict = defaultdict(int)
    homo_key_dict_2 = defaultdict(int)
    homo_dict_save = defaultdict(int)
    count_del = 0
    count_same_wrong = 0
    for key, val in SV_dict_contig_1.items():
        for key_2,val_2 in SV_dict_contig_2.items():
            #print(key)
            chr_num_1 = val[0]
            chr_num_2 = val_2[0]
            start_1 = val[1]
            end_1 = val[2]
            start_2 = val_2[1]
            end_2 = val_2[2]
            ref_1 = val[3]
            ref_2 = val_2[3]
            alt_1 = val[4]
            alt_2 = val_2[4]
            PS_start_1 = int( val[5].split("_PS")[-1].split("_hp1")[0].split(":")[0])
            PS_end_1 = int( val[5].split("_PS")[-1].split("_hp1")[0].split(":")[1])
            PS_start_2 = int( val_2[5].split("_PS")[-1].split("_hp2")[0].split(":")[0])
            PS_end_2 = int( val_2[5].split("_PS")[-1].split("_hp2")[0].split(":")[1])
            if chr_num_1 == chr_num_2 and ((start_1 >= start_2 and end_1 <= end_2) or (start_1 >= start_2 and start_1 <= end_2) or (start_1 <= start_2 and end_1 >= start_2) or (start_1 <= start_2 and end_1 >= end_2)):
                if alt_1 == "-" and alt_2 == "-": # del
                    if key not in homo_key_dict and key_2 not in homo_key_dict_2:
                        if PS_end_1 < PS_start_2 or PS_end_2 < PS_start_1:
                            count_same_wrong += 1
                        else:
                            homo_key_dict[key] = 1
                            homo_key_dict_2[key_2] = 1
                            count_del += 1
                            start_used, end_used = check_start_end([start_1,start_2],[end_1,end_2])
                            sv_len = end_used - start_used
                            contig_num_1 = val[-4]
                            contig_start_1 = val[-3]
                            contig_end_1 = val[-2]
                            contig_strand_dir_1 = val[-1]
                            contig_num_2 = val_2[-4]
                            contig_start_2 = val_2[-3]
                            contig_end_2 = val_2[-2]
                            contig_strand_dir_2 = val_2[-1]
                            del_homo_dict[(chr_num_1,start_used,end_used)] = sv_len
                            #del_homo_sv[(chr_num_1,start_1,end_1)] = [chr_num_1,start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]  # for svviz

                            del_homo_sv[(chr_num_1,start_used,end_used)] = [chr_num_1,start_used,end_used,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2 ]  # for final overlap of 3libs

    # processing the uniq in contig_1
    print(count_del)
    print(len(homo_key_dict))
    print(len(homo_key_dict_2))
    for key, val in SV_dict_contig_1.items():
        if homo_key_dict[key] == 1:
            count_total_share += 1
        else:
            start_1 = val[1]
            end_1 = val[2]
            ref_1 = val[3]
            alt_1 = val[4]
            if alt_1 == "-":  # del
                count_cov = 0
                for _step in range(start_1,end_1 + 1):
                    if _step in ref_dict_contig_2:
                        count_cov += 1
                sv_len_1 = len(ref_1)
                contig_start_1 = val[-3]
                contig_end_1 = val[-2]
                contig_num_1 = val[-4]
                contig_strand_dir_1 = val[-1]

                #if count_cov - 1 == sv_len_1:
                if float(count_cov - 1)/sv_len_1 >= 0.8:
                    del_hetero_dict[key] = sv_len_1
                    del_hetero_sv[key] = [val[0],start_1,end_1,ref_1,alt_1,contig_num_1,contig_start_1,contig_end_1,contig_strand_dir_1,'0_PS0:0_hp0',0,0,0]

    print(count_total_share)
    # processing the uniq in contig_2
    count_total_share = 0
    for key_2, val_2 in SV_dict_contig_2.items():
        if homo_key_dict_2[key_2] == 1:
            count_total_share += 1
        else:
            start_2 = val_2[1]
            end_2 = val_2[2]
            ref_2 = val_2[3]
            alt_2 = val_2[4]
            if alt_2 == "-":  # del
                if _step in range(start_2, end_2+1):
                    if _step in ref_dict_contig_1:
                        count_cov += 1
                sv_len_2 = len(ref_2)
                #if count_cov - 1 == sv_len_2:
                if float(count_cov - 1)/sv_len_2 >= 0.8:
                    del_hetero_dict[key_2] = sv_len_2
                    contig_num_2 = val_2[-4]
                    contig_start_2 = val_2[-3]
                    contig_end_2 = val_2[-2]
                    contig_strand_dir_2 = val_2[-1]
                    del_hetero_sv[key_2] = [val_2[0],start_2,end_2,ref_2,alt_2,'0_PS0:0_hp0',0,0,0,contig_num_2,contig_start_2,contig_end_2,contig_strand_dir_2]

    print(count_total_share)

    print("done")
    print(count_del)
    print("------")
    pickle.dump(del_homo_sv, open(out_dir + "del_homo_sv_chr" + str(chr_num) + ".p","wb"))
    pickle.dump(del_hetero_sv, open(out_dir + "del_hetero_sv_chr" + str(chr_num) + ".p","wb"))

    return count_same_wrong


def Write_vcf_for_SV(fw,del_homo_file,del_hetero_file,count_id):
    del_homo = pickle.load(open(del_homo_file,"rb"))
    del_hetero = pickle.load(open(del_hetero_file,"rb"))
    for key, val in del_homo.items():
        #print(key)
        chr_num = str(val[0])
        start_ = val[1]
        end_ = val[2]
        ref = val[3]
        alt = val[4]
        GT = "1/1"
        contig_num_1 = val[5]
        contig_start_1 = val[6]
        contig_end_1 = val[7]
        contig_strand_dir_1 = val[8]
        contig_num_2 = val[9]
        contig_start_2 = val[10]
        contig_end_2 = val[11]
        contig_strand_dir_2 = val[12]
        new_contig_num_1 = contig_num_1.replace(":","_")
        new_contig_num_2 = contig_num_2.replace(":","_")
        contig_info = str(new_contig_num_1) + "_" + str(contig_start_1) + "_" + str(contig_end_1) + "_" + contig_strand_dir_1 + "_" +  str(new_contig_num_2) + "_" + str(contig_start_2) + "_" + str(contig_end_2) + "_" + contig_strand_dir_2 
        fw.writelines(chr_num + "\t" + str(start_) + "\t" + "event" + str(count_id) + "\t" + ref + "\t" + alt + "\t" + "." + "\t" + "PASS" + "\t" + "SVTYPE=DEL" + "\t" + "GT:Contig" + "\t"  + "1/1" + ":"  + contig_info +  "\n")
        count_id += 1

    for key, val in del_hetero.items():
        #print(key)
        chr_num = str(val[0])
        start_ = val[1]
        end_ = val[2]
        ref = val[3]
        alt = val[4]
        GT = "0/1"
        contig_num_1 = val[5]
        contig_start_1 = val[6]
        contig_end_1 = val[7]
        contig_strand_dir_1 = val[8]
        contig_num_2 = val[9]
        contig_start_2 = val[10]
        contig_end_2 = val[11]
        contig_strand_dir_2 = val[12]
        new_contig_num_1 = contig_num_1.replace(":","_")
        new_contig_num_2 = contig_num_2.replace(":","_")
        contig_info = str(new_contig_num_1) + "_" + str(contig_start_1) + "_" + str(contig_end_1) + "_" + str(contig_strand_dir_1) + "_" +  str(new_contig_num_2) + "_" + str(contig_start_2) + "_" + str(contig_end_2) + "_" + str(contig_strand_dir_2) 
        fw.writelines(chr_num + "\t" + str(start_) + "\t" + "event" + str(count_id) + "\t" + ref + "\t" + alt + "\t" + "." + "\t" + "PASS" + "\t" + "SVTYPE=DEL" + "\t" + "GT:Contig" + "\t"  + "0/1" + ":"  + contig_info +  "\n")
        count_id += 1

 
    return (fw,count_id)






if __name__ == "__main__":
    out_dir = args.out_dir
    if os.path.exists(out_dir):
        print("using existing output folder: " + out_dir)
    else:
        os.makedirs(out_dir)
    chr_start = args.chr_start
    chr_end = args.chr_end
    v_size = args.var_size
    total_same_wrong = 0
    count_id = 1
    for chr_num in range(chr_start,chr_end + 1):
        final_SV_vcf = out_dir + "Aquila_DEL_chr" + str(chr_num) +  ".vcf"
        fw = open(final_SV_vcf,"w")
        SV_dict_contig_1 = Extract_SV_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.var.txt",out_dir + "del_dict_contig_1_chr" + str(chr_num) + ".p", chr_num,v_size)
        SV_dict_contig_2 = Extract_SV_info(out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.var.txt",out_dir + "del_dict_contig_2_chr" + str(chr_num) + ".p", chr_num,v_size)
        count_same_wrong = compare_two_haploid_SV(out_dir + "del_dict_contig_1_chr" + str(chr_num) + ".p", out_dir + "del_dict_contig_2_chr" + str(chr_num) + ".p", chr_num, out_dir + "ref_dict_contig_1_chr" + str(chr_num) + ".p" , out_dir + "ref_dict_contig_2_chr" + str(chr_num) + ".p",out_dir )
        total_same_wrong += count_same_wrong


        fw,count_id = Write_vcf_for_SV(fw,out_dir + "del_homo_sv_chr" + str(chr_num) + ".p", out_dir + "del_hetero_sv_chr" +str(chr_num) + ".p",count_id)
        fw.close()
    print("count_id: " + str(count_id))
    print("total_same_wrong:")
    print(total_same_wrong)



#!/usr/bin/env python
import pdb
#pdb.set_trace()
from collections import defaultdict
import numpy as np
import pickle
from argparse import ArgumentParser
import os
from multiprocessing import Pool,cpu_count,active_children
import time
from subprocess import Popen
parser = ArgumentParser(description="Cut phase block for local assembly:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end at", default=23)
parser.add_argument('--block_threshold','-t',type=int,help="phase block threshold")
parser.add_argument('--block_len_use','-l',type=int,help="phase block len threshold")
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs")
parser.add_argument('--in_dir','-i_dir', help="Directory for inputs")
parser.add_argument('--highconf_profile_dir','-hc_dir', help="Directory for highconf profile")
args = parser.parse_args()


def Finalize_phase_block(output_file_raw, output_file,use_fake_PS_flag):
    PS_flag_dict = defaultdict(int)
    count_mole = defaultdict(int)
    f = open(output_file_raw, "r")
    fw = open(output_file, "w")
    for line in f:
        data = line.rsplit()
        var_dict = []
        if data[-1] == "hp1" or data[-1] == "hp2":
            PS_tag = data[-2]
            PS_flag = int(PS_tag.split(":")[1])
            HP_tag = data[-1]
            if use_fake_PS_flag[PS_flag] == 1:
                var_list = data[7:-2]
                count_ = 0
                for var in var_list:
                    var_pos = int(var.split(":")[0])
                    if count_ == 0:
                        min_pos = var_pos
                    else:
                        if min_pos > var_pos:
                            min_pos = var_pos
                    count_ += 1

                if count_mole[PS_flag] == 0:
                    PS_flag_dict[PS_flag] = min_pos
                else:
                    if min_pos < PS_flag_dict[PS_flag]:
                        PS_flag_dict[PS_flag] = min_pos

                count_mole[PS_flag] += 1

    f.close()
    f = open(output_file_raw, "r")
    for line in f:
        data = line.rsplit()
        if data[-1] == "hp1" or data[-1] == "hp2":
            PS_tag = data[-2]
            PS_flag = int(PS_tag.split(":")[1])
            if use_fake_PS_flag[PS_flag] == 1:
                HP_tag = data[-1]
                data[-2] = "PS:" + str(PS_flag_dict[PS_flag])
                fw.writelines("\t".join(data) + "\n")
            else:
                fw.writelines(line)
        else:
            fw.writelines(line)

    f.close()
    fw.close()


def Check_mole_pos_in_stop_end_list(mole_start,mole_end,stop_end_list):
    count = 0
    use_flags = []
    for stop_end in stop_end_list:
        step = stop_end
        if count > 0:
            if mole_start <= prev_step:
                if mole_end >= prev_step:
                    use_flags.append(count - 1)
            else:
                if mole_start <= step:
                    use_flags.append(count - 1)

        prev_step = step
        count += 1
    return use_flags


def Cut_phaseblock_for_phased_h5(h5_phased_file,chr_num,out_file,block_len_use,block_threshold,output_dir,bed_file,phase_block_file,global_track,HC_breakpoint_file,xin):
    HC_breakpoint = defaultdict(list)
    fw_HC_bk = open(output_dir + "chr" + str(chr_num) + "_HC_breakpoint.bed","w")
    fw_bed_file = open(bed_file,"w")
    count_mole_max = defaultdict(int)
    PS_flag_dict_max = defaultdict(int)
    f = open(h5_phased_file,"r")
    curr = 0
    for line in f:
        curr += 1
        data = line.rsplit()
        mole_len = int(data[3])
        var_dict = []
        if data[-1] == "hp1" or data[-1] == "hp2":
            PS_tag = data[-2]
            PS_flag = int(PS_tag.split(":")[1])
            HP_tag = data[-1]
            var_list = data[7:-2]
            count_ = 0
            for var in var_list:
                var_pos = int(var.split(":")[0])
                if count_ == 0:
                    max_pos = var_pos
                else:
                    if max_pos < var_pos:
                        max_pos = var_pos
                count_ += 1

            if count_mole_max[PS_flag] == 0:
                PS_flag_dict_max[PS_flag] = max_pos
            else:
                if max_pos > PS_flag_dict_max[PS_flag]:
                    PS_flag_dict_max[PS_flag] = max_pos

            count_mole_max[PS_flag] += 1

    f.close()
    all_len = []
    cut_phase_block = defaultdict(int)
    for key, val in PS_flag_dict_max.items():
        len_block = val - key
        all_len.append(len_block)
        fw_bed_file.writelines("chr" + str(chr_num) + "\t" + str(key)+ "\t" + str(val) + "\n")
        if len_block >= block_threshold:
            cut_phase_block[key] = val
      
    #pickle.dump(cut_phase_block,open("cut_phase_block.p","wb"))
    #pickle.dump(PS_flag_dict_max,open("PS_flag_dict_max.p","wb"))
    #cut_phase_block = pickle.load(open("cut_phase_block.p","rb"))
    #PS_flag_dict_max = pickle.load(open("PS_flag_dict_max.p","rb"))

    cut_phase_block_2 = defaultdict(list)
    cut_phase_block_3 = defaultdict(list)
    """  use 100kb for each phase block  """
    global_track_dict =  pickle.load(open(global_track,"rb"))
    count_break_highconf = 0
    count_break_highconf_2 = 0
    count_break_lowconf = 0
    all_cut_block_len = []
    for key, val in cut_phase_block.items():
        cut_phase_block_2[key].append(key)
        phase_block_len = val - key
        num_of_blocks_cut = int(phase_block_len/block_len_use)
        for ii in range(1,num_of_blocks_cut):
            cut_phase_block_2[key].append(key + block_len_use*ii)
        cut_phase_block_2[key].append(val)

        break_point_list = cut_phase_block_2[key]
        cut_phase_block_3[key].append(break_point_list[0])
        for ii in range(1,len(break_point_list)-1):
            break_point = break_point_list[ii]
            break_flag = 0
            if break_point in global_track_dict:
                HC_breakpoint[(key,val)].append(break_point)
                fw_HC_bk.writelines("chr" + str(chr_num) + "\t" + str(break_point) + "\t" + str(break_point+1) + "\n")
                cut_phase_block_3[key].append(break_point)
                count_break_highconf_2 += 1
            else:
                break_point_before = break_point - 5000
                break_point_after = break_point + 5000
                for jj in range(break_point_before, break_point_after+1):
                    if jj in global_track_dict:
                        HC_breakpoint[(key,val)].append(jj)
                        fw_HC_bk.writelines("chr" + str(chr_num) + "\t" + str(jj) + "\t" + str(jj+1) + "\n")
                        cut_phase_block_3[key].append(jj)
                        break_flag = 1
                        count_break_highconf += 1
                        all_cut_block_len.append(jj - cut_phase_block_3[key][-2])
                        break
                if break_flag == 0:
                    cut_phase_block_3[key].append(break_point)
                    count_break_lowconf += 1
                    all_cut_block_len.append(break_point - cut_phase_block_3[key][-2])

        cut_phase_block_3[key].append(break_point_list[-1])                   

    print(np.max(all_cut_block_len))
    print(np.min(all_cut_block_len))
    print(np.mean(all_cut_block_len))
    print(count_break_highconf_2,count_break_highconf,count_break_lowconf)
    count_test_1 = 0
    count_test_2 = 0
    for key, val_list in cut_phase_block_3.items():
        for jj in range(len(val_list)-1):
            val_1 = val_list[jj]
            val_2 = val_list[jj+1]
            count_test_1 += 1
            fw_bed_file.writelines("chr" + str(chr_num) + "\t" + str(val_1)+ "\t" + str(val_2) + "\n")

    for key, val_list in cut_phase_block_2.items():
        for jj in range(len(val_list)-1):
            val_1 = val_list[jj]
            val_2 = val_list[jj+1]
            count_test_2 += 1
            
    fw_bed_file.close()
    temp_file = output_dir + "temp_file_chr" + str(chr_num)
    fw = open(temp_file,"w")
    f = open(h5_phased_file,"r")
    define_PS_flag = defaultdict(lambda: defaultdict(int))
    final_PS_block_cut = defaultdict(int)
    for line in f:
        data = line.rsplit()
        if data[-1] == "hp1" or data[-1] == "hp2":
            PS_tag = data[-2]
            PS_flag = int(PS_tag.split(":")[1])
            HP_tag = data[-1]
            if PS_flag in cut_phase_block_3.keys(): # if the phase block length >= threshold_length
                stop_end_list = cut_phase_block_3[PS_flag]
                mole_start = int(data[1])
                mole_end = int(data[2])
                use_flags = Check_mole_pos_in_stop_end_list(mole_start,mole_end,stop_end_list)
                if use_flags == []:
                    use_flags = [0]
                for _pos in use_flags:
                    if _pos == 0:  # keep the first block PS flag
                        fw.writelines(line)
                        final_PS_block_cut[PS_flag] = stop_end_list[_pos+1]
                   
                    else:  # reassign the fake phase block number for the rest
                        cur_PS_flag = stop_end_list[_pos]
                        data[-2] = "PS:" + str(cur_PS_flag)
                        final_PS_block_cut[cur_PS_flag] = stop_end_list[_pos+1]
                        fw.writelines("\t".join(data) + "\n")
            else:
                fw.writelines(line)
                final_PS_block_cut[PS_flag] = PS_flag_dict_max[PS_flag]
        else:
            fw.writelines(line)

    fw.close()
    f.close()
    fw_HC_bk.close()
    if os.path.exists(out_file):
        Popen("rm " + out_file,shell=True).wait()
    Popen("mv " + temp_file + " " + out_file,shell=True).wait()
    pickle.dump(final_PS_block_cut,open(phase_block_file,"wb"))
    pickle.dump(HC_breakpoint,open(HC_breakpoint_file,"wb"))
    print("done~")    





if __name__ == "__main__":
    input_dir = args.in_dir
    hc_dir = args.highconf_profile_dir
    output_dir = args.out_dir
    if os.path.exists(output_dir):
        print("using existing output folder: " + output_dir)
    else:
        os.makedirs(output_dir)
    chr_start = args.chr_start
    chr_end = args.chr_end
    block_len_use = args.block_len_use
    block_threshold = args.block_threshold
    #pool = Pool(chr_end - chr_start + 1)
    for chr_num in range(chr_start,chr_end+1):
        file_name = input_dir + "chr" + str(chr_num) + ".phased_final"
        out_file = output_dir + "chr" + str(chr_num) + ".phased_final_cut_by_" + str(block_len_use) 
        bed_file = output_dir + "chr" + str(chr_num) + ".phased_final_cut_by_" + str(block_len_use) + "_phase_blocks.bed"
        phase_block_file = output_dir + "chr" + str(chr_num) + ".phased_final_cut_by_" + str(block_len_use) + "_phase_blocks.p"
        HC_breakpoint_file = output_dir + "chr" + str(chr_num) + ".phased_final_cut_by_" + str(block_len_use) + "_HC_breakpoint_2.p"
        global_track = hc_dir + "chr" + str(chr_num) + "_global_track.p"
        Cut_phaseblock_for_phased_h5(file_name,chr_num,out_file,block_len_use,block_threshold,output_dir,bed_file,phase_block_file,global_track,HC_breakpoint_file,"xin")
        #pool.apply_async(Cut_phaseblock_for_phased_h5,(file_name,chr_num,out_file,block_len_use,block_threshold,output_dir,bed_file,phase_block_file,global_track,HC_breakpoint_file,"xin"))
   
    """
    pool.close()
    while len(active_children()) > 1:
        time.sleep(0.5)
    pool.join()
    """
    print("all done~")





#!/usr/bin/env python
import pdb
#pdb.set_trace()
import glob
import os
import numpy as np
from argparse import ArgumentParser
from collections import defaultdict
import time
import multiprocessing as mp
import pickle
import sys
import pysam
from subprocess import Popen
from multiprocessing import Pool,cpu_count,active_children
parser = ArgumentParser(description="Extract reads for all small chunks:")
parser.add_argument('--bam_file','-bam',help="bam file")
parser.add_argument('--vcf_file','-v',help="vcf file")
parser.add_argument('--mbq_threshold','-bq',type=int,help="mbq threshold", default=13)
parser.add_argument('--mmq_threshold','-mq',type=int,help="mmq threshold", default=20)
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--boundary','-b',type=int,help="cut boundary", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--num_threads','-n',type=int,help="number of threads", default=8)
parser.add_argument('--sample_name','-s',help="sample name")
parser.add_argument('--out_dir','-o',help="output folder")

args = parser.parse_args()
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 


def flatten(listoflist):
    flat_list = []
    for alist in listoflist:
        for val in alist:
            flat_list.append(val)
    return flat_list


def Cal_snp_ratio_vs_depth(vcf_file,mbq_threshold):
    f = open(vcf_file,"r")
    curr = 0
    variant_dp_dict =  defaultdict(int)
    curr = 0
    count_total = 0
    total_depth = []
    for line in f:
        data = line.rsplit()
        curr += 1
        if data[0][:2] != "##" and data[0][:2] != "#C":
            try:
                qual = float(data[5])
                chr_num = int(data[0][3:])
                pos = int(data[1]) - 1
                ref = data[3]
                alt = data[4]
                GT = data[9].split(":")[0]
                if (GT == "0/1"  or GT == "1/0" or GT == "0|1" or GT == "1|0") and len(ref) == 1 and len(alt) == 1 and qual >= mbq_threshold:
                    break_flag = 0
                    _format = data[8].split(":")
                    AO_idx = _format.index("AO")
                    RO_idx = _format.index("RO")
                    DP_idx = _format.index("DP")
                    _format_info = data[9].split(":")
                    ao_depth = float(_format_info[AO_idx])
                    ro_depth = float(_format_info[RO_idx])
                    dp_depth = float(_format_info[DP_idx])
                    if ao_depth >= ro_depth:
                        ratio = ro_depth/ao_depth
                    else:
                        ratio = ao_depth/ro_depth
                    for ii in range(0,11):
                        if dp_depth <= 10*(ii + 1) and dp_depth > 10*ii: 
                            for jj in range(0,20):
                                if ratio <= 0.05*(jj+1) and ratio > 0.05*jj:
                                    variant_dp_dict[(ii,jj)] += 1
                                    count_total += 1
                                    break_flag = 1
                                    total_depth.append(dp_depth)
                                    break
                    if break_flag == 0:
                        if dp_depth > 110:
                            for jj in range(0,20):
                                if ratio <= 0.05*(jj+1) and ratio > 0.05*jj:
                                    variant_dp_dict[(11,jj)] += 1
                                    count_total += 1
                                    total_depth.append(dp_depth)
            except:
                if data[0] == "chrX":
                    chr_num = "X"
                qual = float(data[5])
                pos = int(data[1]) - 1
                ref = data[3]
                alt = data[4]
                GT = data[9].split(":")[0]
                if (GT == "0/1"  or GT == "1/0" or GT == "0|1" or GT == "1|0") and len(ref) == 1 and len(alt) == 1 and qual >= mbq_threshold:
                    break_flag = 0
                    _format = data[8].split(":")
                    AO_idx = _format.index("AO")
                    RO_idx = _format.index("RO")
                    DP_idx = _format.index("DP")
                    _format_info = data[9].split(":")
                    ao_depth = float(_format_info[AO_idx])
                    ro_depth = float(_format_info[RO_idx])
                    dp_depth = float(_format_info[DP_idx])
                    if ao_depth >= ro_depth:
                        ratio = ro_depth/ao_depth
                    else:
                        ratio = ao_depth/ro_depth
                    for ii in range(0,11):
                        if dp_depth <= 10*(ii + 1) and dp_depth > 10*ii: 
                            for jj in range(0,20):
                                if ratio <= 0.05*(jj+1) and ratio > 0.05*jj:
                                    variant_dp_dict[(ii,jj)] += 1
                                    count_total += 1
                                    break_flag = 1
                                    total_depth.append(dp_depth)
                                    break
                    if break_flag == 0:
                        if dp_depth > 110:
                            for jj in range(0,20):
                                if ratio <= 0.05*(jj+1) and ratio > 0.05*jj:
                                    variant_dp_dict[(11,jj)] += 1
                                    count_total += 1
                                    total_depth.append(dp_depth)


    #pickle.dump(variant_dp_dict,open("variant_dp_dict.p","wb"))
    avg_depth = np.mean(total_depth)
    median_depth = np.median(total_depth)
    print(count_total)
    print(np.mean(total_depth))
    print(np.median(total_depth))
    print(np.max(total_depth))
    print(np.min(total_depth))
    return (avg_depth,median_depth)


def save_variant_dict(vcf_file,mbq_threshold,avg_depth,out_dir):
    f = open(vcf_file,"r")
    variant_dict =  defaultdict(list)
    for line in f:
        data = line.rsplit()
        if data[0][:2] != "##" and data[0][:2] != "#C":
            try:
                qual = float(data[5])
                chr_num = int(data[0][3:])
                pos = int(data[1]) - 1
                ref = data[3]
                alt = data[4]
                GT = data[9].split(":")[0]
                if (GT == "0/1"  or GT == "1/0") and len(ref) == 1 and len(alt) == 1 and qual >= mbq_threshold:
                    _format = data[8].split(":")
                    AO_idx = _format.index("AO")
                    RO_idx = _format.index("RO")
                    DP_idx = _format.index("DP")
                    _format_info = data[9].split(":")
                    ao_depth = float(_format_info[AO_idx])
                    ro_depth = float(_format_info[RO_idx])
                    dp_depth = float(_format_info[DP_idx])
                    if ao_depth >= ro_depth:
                        ratio = ro_depth/ao_depth
                    else:
                        ratio = ao_depth/ro_depth
                    if ratio >= 0.25 and dp_depth >= 0.1*avg_depth and dp_depth <= 0.9* avg_depth:
                        variant_dict[(chr_num,pos)] = [ref,alt,GT]
            except:
                if data[0] == "chrX":
                    chr_num = "X"
                qual = float(data[5])
                pos = int(data[1]) - 1
                ref = data[3]
                alt = data[4]
                GT = data[9].split(":")[0]
                if (GT == "0/1"  or GT == "1/0") and len(ref) == 1 and len(alt) == 1 and qual >= mbq_threshold:
                    _format = data[8].split(":")
                    AO_idx = _format.index("AO")
                    RO_idx = _format.index("RO")
                    DP_idx = _format.index("DP")
                    _format_info = data[9].split(":")
                    ao_depth = float(_format_info[AO_idx])
                    ro_depth = float(_format_info[RO_idx])
                    dp_depth = float(_format_info[DP_idx])
                    if ao_depth >= ro_depth:
                        ratio = ro_depth/ao_depth
                    else:
                        ratio = ao_depth/ro_depth
                    if ratio >= 0.25 and dp_depth >= 0.1*avg_depth and dp_depth <= 0.9* avg_depth:
                        variant_dict[(chr_num,pos)] = [ref,alt,GT]
       
    pickle.dump(variant_dict, open(out_dir + "variant_dict_heterozygous.p", "wb"))
    return variant_dict     


def get_match_num(cigar):
    used_num = ""
    cumu_num = 0
    for i in range(len(cigar)):
        cur_num = cigar[i]
        try:
            first_num = int(cur_num)
            used_num += cur_num
        except ValueError:
            if cur_num != "M":
                cumu_num += int(used_num)
                used_num = ""
            else:
                match_num = int(used_num)
                break
    return (cumu_num,match_num)


def get_match_num_revised(cigar):
    cigar_len = len(cigar)
    cigar_list = []
    num_string = ""
    for i in range(cigar_len):
        letter = cigar[i]
        if letter.isalpha():
            cigar_list.append(num_string)
            num_string = ""
            cigar_list.append(letter)
        else:
            num_string += letter

    indices = [i for i, x in enumerate(cigar_list) if x == "M"]
    cumu_num = 0
    cumu_start = 0
    cumu_num_list = []
    cumu_start_list = []
    match_num_list = []
    for idx in indices:
        match_num = int(cigar_list[idx - 1])
        match_num_list.append(match_num)
        for i in range(0,idx,1):
            parameter = cigar_list[i+1]
            if parameter in ["M", "S", "H", "I"]:
                cumu_num += int(cigar_list[i])
            if parameter in ["M", "D"]:
                cumu_start += int(cigar_list[i])

        cumu_num_list.append(cumu_num-match_num)
        cumu_start_list.append(cumu_start-match_num)
        cumu_num = 0
        cumu_start = 0

    return (cumu_num_list,cumu_start_list,match_num_list)


def check_read_hp(hp,barcode):
    if len(set(hp)) == 1:
        hp_use = True
        hp_flag = hp[0]
    else:
        count0 = hp.count("0")
        count1 = hp.count("1")
        if count0 >= 4*count1:
            hp_use = True
            hp_flag = "0"
        elif count1 >= 4*count0:
            hp_use = True
            hp_flag = "1"
        else:
            hp_use = False
            hp_flag = "none"
    return (hp_use,hp_flag)


def get_mole_variant(chr_num,mole_dict_3,mole_dict_4,mole_dict_5,barcode):
    mole_variant = defaultdict(list)
    for qname,cigar_all in mole_dict_5[barcode].items():
        for ii in range(len(mole_dict_5[barcode][qname])):
            cigar = cigar_all[ii]
            if cigar != None:
                curr_string =  mole_dict_4[barcode][qname][ii]
                cumu_num, match_num = get_match_num(cigar)
                start_locus = int(mole_dict_3[barcode][qname][ii])
                match_string = curr_string[cumu_num:cumu_num+match_num]
                end_locus = len(match_string) - 1
                for jj in range(0,end_locus-1):
                    idx_j = jj + start_locus - 1
                    val = variant_dict[(chr_num,idx_j)]
                    if val != []:
                        ref = val[0]
                        alt = val[1]
                        GT = val[2]
                        match_str = match_string[jj:jj+len(ref)]
                        if match_str == ref:
                            mole_variant[idx_j].append("0")
                        elif match_str == alt:
                            mole_variant[idx_j].append("1")

    return mole_variant


def get_mole_variant_revised(chr_num,mole_dict_3,mole_dict_4,mole_dict_5,barcode):
    mole_variant = defaultdict(list)
    for qname,cigar_all in mole_dict_5[barcode].items():
        for ii in range(len(mole_dict_5[barcode][qname])):
            cigar = cigar_all[ii]
            if cigar != None:
                curr_string =  mole_dict_4[barcode][qname][ii]
                cumu_num_list,cumu_start_list, match_num_list = get_match_num_revised(cigar)
                for kk in range(len(cumu_num_list)):
                    cumu_num = cumu_num_list[kk]
                    cumu_start = cumu_start_list[kk]
                    match_num = match_num_list[kk]
                    start_locus = int(mole_dict_3[barcode][qname][ii]) + cumu_start
                    match_string = curr_string[cumu_num:cumu_num+match_num]
                    end_locus = len(match_string) - 1
                    for jj in range(0,end_locus-1):
                        idx_j = jj + start_locus - 1
                        val = variant_dict[(chr_num,idx_j)]
                        if val != []:
                            ref = val[0]
                            alt = val[1]
                            GT = val[2]
                            match_str = match_string[jj:jj+len(ref)]
                            if match_str == ref:
                                mole_variant[idx_j].append("0")
                                #if kk > 0:
                                    #print("use here", ref, alt, match_str,match_num, match_num_list[0])
                            elif match_str == alt:
                                mole_variant[idx_j].append("1")
                                #if kk > 0:
                                    #print("use here", ref, alt, match_str,match_num, match_num_list[0])
                            #else:
                                #if kk > 0:
                                    #print("wrong here",ref,alt,match_str)

    return mole_variant


def save_pickle_file(dict1,fname):
    for value in dict1:
        dict1[value] = dict(dict1[value])
    my_dict = dict(dict1)
    with open(fname, "wb") as f:
        pickle.dump(my_dict,f) 


def process_sorted_bam(bam_file,output_file,qname_file,qname_pos_file,variant_dict,threshold_start,threshold_end,boundary,mmq_threshold,xin):
    #qname_pos = defaultdict(int)
    qname_pos = defaultdict(list)
    mole_qname_dict = defaultdict(lambda: defaultdict(list))
    sam_file = pysam.AlignmentFile(bam_file, "rb")
    threshold_pairs = 2
    fw= open(output_file,"w")
    mole_dict = defaultdict(list)
    mole_dict_2 = defaultdict(lambda: defaultdict(list))
    mole_dict_3 = defaultdict(lambda: defaultdict(list))
    mole_dict_4 = defaultdict(lambda: defaultdict(list))   # read string
    mole_dict_5 = defaultdict(lambda: defaultdict(list))   # cigar
    curr = 0
    count_mole = 1
    chr_begin = threshold_start
    if threshold_start == 23:
        use_chr_num = "chrX"
    else:
        use_chr_num = "chr" + str(threshold_start)
    for read in sam_file.fetch(use_chr_num):
        curr += 1
        raw_chr_num = read.reference_name
        if use_chr_num == "chrX":
            chr_num = "X"
        else:
            chr_num = int(read.reference_name[3:])
        tags = read.get_tags()
        barcode_field = [s for s in tags if "BX" in s]
        mq = read.mapping_quality
        if barcode_field != [] and mq >= mmq_threshold:
            barcode =  barcode_field[0][1].split("-")[0]
            start_pos = read.pos + 1   # use "1" coordinate
            qname = read.qname
            if read.is_read1:
                read_pair = 1
            elif read.is_read2:
                read_pair = 2
            #qname_pos[(qname,read_pair)] = start_pos
            qname_pos[qname].append(start_pos)
            if len(mole_dict[barcode]) == 0:
                mole_dict[barcode].append(start_pos) 
                mole_dict_2[barcode][qname].append(read_pair)
                mole_dict_3[barcode][qname].append(start_pos) 
                mole_dict_4[barcode][qname].append(read.seq) 
                mole_dict_5[barcode][qname].append(read.cigarstring) 
            elif len(mole_dict[barcode]) > 0:
                dist = start_pos - mole_dict[barcode][-1]
                if dist < boundary:   
                    mole_dict[barcode].append(start_pos) 
                    mole_dict_2[barcode][qname].append(read_pair)
                    mole_dict_3[barcode][qname].append(start_pos)
                    mole_dict_4[barcode][qname].append(read.seq)
                    mole_dict_5[barcode][qname].append(read.cigarstring)
                else:
                    count_del = 0
                    for key,value in mole_dict_2[barcode].items():
                        if len(value) == 1:
                            count_del += 1

                    all_pos = mole_dict_3[barcode].values()
                    num_of_pairs = len(all_pos) - count_del

                    if num_of_pairs >= threshold_pairs :  # 2 pairs
                        poss = flatten(all_pos)
                        start_ = min(poss)
                        end_ = max(poss)
                        mole_len = end_ - start_  + 150
                        mole_qname_dict[count_mole] = mole_dict_2[barcode].copy()
                        mole_variant = get_mole_variant_revised(chr_num,mole_dict_3,mole_dict_4,mole_dict_5,barcode)
                        if mole_variant == {}:
                            fw.write(str(chr_num) + "\t" + str(start_) + "\t" + str(end_) + "\t" + str(mole_len) + "\t" + str(len(poss)) + "\t" + str(barcode) + "\t" + str(count_mole) + "\n")
                        else:
                            fw.write(str(chr_num) + "\t" + str(start_) + "\t" + str(end_) + "\t" + str(mole_len) + "\t" + str(len(poss)) + "\t" + str(barcode) + "\t" + str(count_mole) + "\t")
                            count_var = 0
                            for locus_start, hp in mole_variant.items():
                                hp_use, hp_flag = check_read_hp(hp,barcode)
                                if count_var == len(mole_variant) - 1:
                                    if hp_use:
                                        fw.write(str(locus_start) + ":" + hp_flag + "\n")
                                        #count_1 += 1
                                    else:
                                        fw.write("\n")
                                        #count_2 += 1
                                else:
                                    if hp_use:
                                        fw.write(str(locus_start) + ":" + hp_flag + "\t")
                                        #count_1 += 1
                                    #else:
                                        #count_2 += 1
                                count_var += 1

                        mole_dict[barcode] = [] 
                        mole_dict_2[barcode] = defaultdict(list)
                        mole_dict_3[barcode] = defaultdict(list) 
                        mole_dict_4[barcode] = defaultdict(list) 
                        mole_dict_5[barcode] = defaultdict(list) 
                        mole_dict[barcode].append(start_pos)
                        mole_dict_2[barcode][qname].append(read_pair)
                        mole_dict_3[barcode][qname].append(start_pos)
                        mole_dict_4[barcode][qname].append(read.seq)
                        mole_dict_5[barcode][qname].append(read.cigarstring)
                        count_mole += 1
                    else:
                        mole_dict[barcode] = []
                        mole_dict_2[barcode] = defaultdict(list)
                        mole_dict_3[barcode] = defaultdict(list)
                        mole_dict_4[barcode] = defaultdict(list)
                        mole_dict_5[barcode] = defaultdict(list)
                        mole_dict[barcode].append(start_pos)
                        mole_dict_2[barcode][qname].append(read_pair)
                        mole_dict_3[barcode][qname].append(start_pos)
                        mole_dict_4[barcode][qname].append(read.seq)
                        mole_dict_5[barcode][qname].append(read.cigarstring)


    if threshold_end == 23:
        chr_num = "X"
    else:
        chr_num = threshold_end

    for barcode,value in mole_dict.items():
        if len(value) > 0:
            count_del = 0
            for qname,num_reads in mole_dict_2[barcode].items():
                if len(num_reads) == 1:
                    count_del += 1

            all_pos = mole_dict_3[barcode].values()
            num_of_pairs = len(all_pos) - count_del
            if num_of_pairs >= threshold_pairs : # 2 pairs
                poss = flatten(all_pos)
                start_ = min(poss)
                end_ = max(poss)
                mole_len = end_ - start_  + 150
                mole_qname_dict[count_mole] = mole_dict_2[barcode].copy()
                mole_variant = get_mole_variant_revised(chr_num,mole_dict_3,mole_dict_4,mole_dict_5,barcode)
                if mole_variant == {}:
                    fw.write(str(chr_num) + "\t" + str(start_) + "\t" + str(end_) + "\t" + str(mole_len) + "\t" + str(len(poss)) + "\t" + str(barcode) + "\t" + str(count_mole) + "\n")
                else:
                    fw.write(str(chr_num) + "\t" + str(start_) + "\t" + str(end_) + "\t" + str(mole_len) + "\t" + str(len(poss)) + "\t" + str(barcode) + "\t" + str(count_mole) + "\t")
                    count_var = 0
                    for locus_start, hp in mole_variant.items():
                        hp_use, hp_flag = check_read_hp(hp,barcode)
                        if count_var == len(mole_variant) - 1:
                            if hp_use:
                                fw.write(str(locus_start) + ":" + hp[0] + "\n")
                                #count_1 += 1
                            else:
                                fw.write("\n")
                                #count_2 += 1
                        else:
                            if hp_use:
                                fw.write(str(locus_start) + ":" + hp[0] + "\t")
                                #count_1 += 1
                            #else:
                                #count_2 += 1
                        count_var += 1

                count_mole += 1

    
    fw.close()
    sam_file.close()
    save_pickle_file(mole_qname_dict,qname_file)
    pickle.dump(qname_pos,open(qname_pos_file,"wb"))
    mole_qname_dict.clear()
    qname_pos.clear()



def Run_h5_all(bam_file,vcf_file,chr_start,chr_end,sample_name,mbq_threshold,mmq_threshold,boundary,num_threads,out_dir,variant_dict):
    count = 1
    round_times = int(np.ceil((chr_end - chr_start + 1)/num_threads))
    old_list = defaultdict(list)
    for ii in range(0,num_threads):
        for jj in range(chr_start + round_times*ii,chr_start + round_times*ii + round_times):
            if jj <= chr_end:
                old_list[ii+1].append(jj)

    new_list = defaultdict(list)
    times = 1
    for key,val in old_list.items():
        if times%2 == 1:
            new_list[key] = val
        elif times%2 == 0:
            new_list[key] = val[::-1] 
        times += 1
    
    run_list = defaultdict(list)
    for key,val in new_list.items():
        for ii in range(round_times):
            if ii < len(val):
                run_list[ii+1].append(val[ii])

    for key, val in run_list.items():
        pool = mp.Pool(processes=num_threads)
        for chr_num in val:
            print("processing chr" + str(chr_num))
            h5_file = out_dir + sample_name + "_chr" + str(chr_num)
            h5_qname_file = out_dir + sample_name + "_chr" + str(chr_num) + "_qname.p"
            h5_qname_pos_file = out_dir + sample_name + "_chr" + str(chr_num) + "_qname_pos.p"
            pool.apply_async(process_sorted_bam,(bam_file,h5_file,h5_qname_file,h5_qname_pos_file,variant_dict,chr_num,chr_num,boundary,mmq_threshold,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()
        print("done here")

    # sort h5 file by coordinates
    for chr_num in range(chr_start,chr_end + 1):
        one_h5 = out_dir + sample_name + "_chr" + str(chr_num)
        one_h5_sorted = out_dir + sample_name + "_chr" + str(chr_num) + "_sorted.h5"
        sort_cmd = "cat " + one_h5 + " | sort -k2n > " + one_h5_sorted
        Popen(sort_cmd,shell=True).wait()
    print("All Done~")




if __name__ == "__main__":
    out_dir = args.out_dir
    if os.path.exists(out_dir):
        print("using existing output folder: " + out_dir)
    else:
        os.makedirs(out_dir)
    bam_file = args.bam_file
    vcf_file = args.vcf_file
    mbq_threshold = args.mbq_threshold
    mmq_threshold = args.mmq_threshold
    boundary = args.boundary
    sample_name = args.sample_name
    avg_depth,median_depth = Cal_snp_ratio_vs_depth(vcf_file,mbq_threshold)
    variant_dict = save_variant_dict(vcf_file,mbq_threshold,avg_depth,out_dir)
    var_depth_file = out_dir + "median_depth_for_var.txt"
    if ~os.path.isfile(var_depth_file):
        Popen("touch " + var_depth_file,shell=True).wait()
        f = open(var_depth_file,"w")
        f.writelines(str(median_depth) + "\n")
        f.close()
    #variant_dict = pickle.load(open("variant_dict_heterozygous.p","wb"))
    chr_start = args.chr_start
    chr_end = args.chr_end
    num_threads = args.num_threads
    Run_h5_all(bam_file,vcf_file,chr_start,chr_end,sample_name,mbq_threshold,mmq_threshold,boundary,num_threads,out_dir,variant_dict)



#!/usr/bin/env python
import pdb
#pdb.set_trace()
from collections import defaultdict
from multiprocessing import Pool,cpu_count,active_children,Manager
import time
import pickle
import os
import numpy as np
from Assign_phase_block_v3 import * 
import sys
from scipy.special import comb
import glob
from Molecule_phase_alg2_withProbModel_v3_MT2_hybrid import *
from argparse import ArgumentParser
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 

parser = ArgumentParser(description="phase molecules:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--overlap_threshold','-t1',type=int,help="overlap threshold", default=3)
parser.add_argument('--support_threshold','-t2',type=int,help="support threshold", default=5)
parser.add_argument('--sample_name','-s',help="sample name")
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs")
parser.add_argument('--h5_dir','-h_dir', help="Directory to store h5 file")

args = parser.parse_args()


def Run_phasing_all(chr_start,chr_end,overlap_threshold,support_threshold,output_dir,h5_dir,sample_name):
    pool = Pool(chr_end - chr_start + 1)
    for chr_num in range(chr_start,chr_end+1):
        pool.apply_async(Phase_start,(output_dir,h5_dir,sample_name,chr_num,chr_num,overlap_threshold,support_threshold,"xin"))
        #Phase_start(output_dir,h5_dir,sample_name,chr_num,chr_num,overlap_threshold,support_threshold,"xin")
    pool.close()
    while len(active_children()) > 1:
        time.sleep(0.5)
    pool.join()
    print("All Done~")




if __name__ == "__main__":
    output_dir = args.out_dir
    h5_dir = args.h5_dir
    if os.path.exists(output_dir):
        print("using existing output folder: " + output_dir)
    else:
        os.makedirs(output_dir)
    overlap_threshold = args.overlap_threshold
    support_threshold = args.support_threshold
    chr_start = args.chr_start
    chr_end = args.chr_end
    sample_name = args.sample_name
    Run_phasing_all(chr_start,chr_end,overlap_threshold,support_threshold,output_dir,h5_dir,sample_name)



#!/usr/bin/env python
from subprocess import Popen
from argparse import ArgumentParser
import os
import sys
from multiprocessing import Pool,cpu_count,active_children,Manager
import time
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
__author__ = "Xin Zhou@Stanford"

parser = ArgumentParser(description="sort bam by qname:")
parser.add_argument('--bam_file_list','-bam',help="Required Parameter, BAM file list, each BAM file is seperately by comma \",\". For example: 1.bam,2.bam",required=True)
parser.add_argument('--out_dir','-o', help="Directory to store Aquila assembly results, default =./Assembly_results", default="./Asssembly_results")
parser.add_argument('--sample_name_list','-sl', help='Required Parameter, The sample names list, each sample name is seperately by comma ",". For example: S24385_lysis_2,S24385_lysis_2H', type=str,required=True)
parser.add_argument('--num_threads_for_samtools_sort','-t',type=int, help=" The number of threads you can define for samtoos sort, default = 20",default=20)
args = parser.parse_args()

def sort_start(sort_bam_cmd,idx_bam_cmd,xin):
    Popen(sort_bam_cmd,shell=True).wait()
    Popen(idx_bam_cmd,shell=True).wait()


if __name__ == "__main__":
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_sortbam_hybrid.py -h",shell=True).wait()
    else:
        sample_list = [item for item in args.sample_name_list.split(',')]
        bam_list = [item for item in args.bam_file_list.split(',')]
        out_dir = args.out_dir
        num_threads = int(args.num_threads_for_samtools_sort)
        if os.path.exists(out_dir):
            print("using existing output folder: " + out_dir)
        else:
            os.makedirs(out_dir)

        count = 1
        total_num = len(sample_list)
        pool = Pool(processes=total_num)
        for ii in range(len(sample_list)):
            sample_name = sample_list[ii]
            bam_file = bam_list[ii]
            bam_file_dir = out_dir + "/sorted_bam_" + sample_name + "/"
            if os.path.exists(bam_file_dir):
                print("using existing output folder: " + bam_file_dir)
            else:
                os.makedirs(bam_file_dir)

            bam_sorted_file = bam_file_dir + "sorted_bam.bam"
            try:
                sort_bam_cmd = "samtools sort -@ " + str(num_threads) + " -n " + bam_file + " -o " + bam_sorted_file
            except:
                sort_bam_cmd = code_path + "samtools/" + "samtools sort -@ " + str(num_threads) + " -n " + bam_file + " -o " + bam_sorted_file
                
            idx_bam_cmd = "touch "  + bam_file_dir + "finish_bam.txt"
            pool.apply_async(sort_start,(sort_bam_cmd,idx_bam_cmd,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()
        print("Sorting bam finished...")



import pdb
#pdb.set_trace()
from argparse import ArgumentParser
import os
import sys
import time
from multiprocessing import Pool,cpu_count,active_children,Manager
from subprocess import Popen
import glob
__author__ = "Xin Zhou@Stanford"
parser = ArgumentParser(description="Clean Assembly Data by Aquila:")
parser.add_argument('--assembly_dir','-i', help="assembly folder",required=True)
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--num_of_threads','-nt', help="number of threads",default= 20)
args = parser.parse_args()

def del_one_file(one_file,xin):
    rm_cmd = "rm -rf " + one_file
    Popen(rm_cmd,shell=True).wait()


def Aquila_clean(in_dir,chr_start,chr_end,num_of_threads):
    local_assembly_dir = in_dir + "Local_Assembly_by_chunks/"
    for chr_num in range(chr_start,chr_end + 1):
        one_dir = local_assembly_dir + "chr" + str(chr_num)  + "_files_cutPBHC/"
        all_files = sorted(glob.glob(one_dir +  "fastq_by_*_assembly"),key=os.path.getsize,reverse=True)
        total_num = len(all_files)
        pool = Pool(num_of_threads)
        count = 1
        for one_file in all_files:
            count += 1
            pool.apply_async(del_one_file,(one_file,"xin"))
            if (count - 1)%num_of_threads == 0 or (count - 1) == total_num:
                pool.close()
                while len(active_children()) > 1:
                    time.sleep(0.5)
                pool.join()
                if (count - 1) == total_num:
                    print("finished chr" + str(chr_num))
                else:
                    pool = Pool(num_of_threads)
        rm_cmd = "rm -rf " + one_dir  + "*.fastq"
        Popen(rm_cmd,shell=True).wait()

    print("Cleaning local assembly by chunks files done!")

    Raw_fastqs_dir = in_dir + "Raw_fastqs/"
    rm_cmd = "rm -rf " + Raw_fastqs_dir
    Popen(rm_cmd,shell=True).wait()
    print("Cleaning Raw_fastqs done!")

    bam_dir = in_dir + "sorted_bam/"
    rm_cmd = "rm -rf " + bam_dir
    Popen(rm_cmd,shell=True).wait()
    print("Cleaning sorted_bam done!")

    hc_dir = in_dir + "HighConf_file/"
    rm_cmd_1 = "rm -rf " + hc_dir + "*.bam" 
    rm_cmd_2 = "rm -rf " + hc_dir + "*.txt" 
    rm_cmd_3 = "rm -rf " + hc_dir + "*pos_cf.p" 
    Popen(rm_cmd_1,shell=True).wait()
    Popen(rm_cmd_2,shell=True).wait()
    Popen(rm_cmd_3,shell=True).wait()
    print("Cleaning HighConf file done!")
 


if __name__ == "__main__":
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_clean.py -h",shell=True).wait()
    else:
        in_dir = args.assembly_dir + "/"
        num_of_threads = int(args.num_of_threads)
        chr_start = args.chr_start
        chr_end = args.chr_end
        Aquila_clean(in_dir,chr_start,chr_end,num_of_threads)





import pdb
#pdb.set_trace()
from collections import defaultdict
from argparse import ArgumentParser
import pickle
from subprocess import Popen
import sys
import os
parser = ArgumentParser(description="Get Uniqness_map folder to run Aquila:")
parser.add_argument('--umap_bed_gz_file','-i',help="Required parameter, k100.umap.bed.gz by hoffmanMappability, k100 is recommended, you could also use k24,k36,k50",required=True)
parser.add_argument('--chr_start','-start',type=int,help="chromosome number start from, default = 1 ", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome number end by, default = 23 ", default=23)
parser.add_argument('--out_dir','-o_dir', help="Directory to store uniqness map files, default = ./Uniqness_map", default='Uniqness_map')

args = parser.parse_args()


def Get_uniqness(input_file,output_file):
    uniq_map = defaultdict(int)
    f = open(input_file,"r")
    for line in f:
        data = line.rsplit()
        _start = int(data[1])
        _end = int(data[2])
        block_len = _end - _start
        if block_len >= 500:
            use_start = _start + 10
            use_end = _end - 10
            for step in range(use_start, use_end+1):
                uniq_map[step] = 1
    pickle.dump(uniq_map,open(output_file,"wb"))




if __name__ == "__main__":
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_sortbam.py -h",shell=True).wait()
    else:
        umap_bed_gz = args.umap_bed_gz_file
        output_file_prefix = "uniq_map"
        out_dir = args.out_dir + "/"
        chr_start = args.chr_start
        chr_end = args.chr_end
        if os.path.exists(out_dir):
            print("using existing output folder: " + out_dir)
        else:
            os.makedirs(out_dir)
        for chr_num in range(chr_start,chr_end + 1):
            uniq_file = out_dir +  "chr" + str(chr_num) + "_uniq.bed"
            if chr_num == 23:
                run_cmd = "zcat " + umap_bed_gz + "| grep -w \"chrX\""  + " > " + uniq_file
            else:
                run_cmd = "zcat " + umap_bed_gz + "| grep -w \"chr\"" + str(chr_num) + " > " + uniq_file
            Popen(run_cmd,shell=True).wait()
            output_file = out_dir + output_file_prefix + "_chr" + str(chr_num) + ".p"
            Get_uniqness(uniq_file,output_file)





import pdb
#pdb.set_trace()
import os
from argparse import ArgumentParser
from subprocess import Popen, PIPE
from multiprocessing import Pool,cpu_count,active_children,Manager
import time
import sys
parser = ArgumentParser(description="Run depth all:")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from, default = 1", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by, default = 23", default=23)
parser.add_argument('--var_size','-v',type=int,help="variant size, cut off size for indel and SV, default = 1", default=1)
parser.add_argument('--num_of_threads','-t',type=int,help="number of threads, default = 1", default=1)
parser.add_argument('--assembly_dir','-i_dir', help="Required parameter, folder to store Aquila assembly results at Aquila assembly steps",required=True)
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs, default = ./Aquila_Variant_Results", default='Aquila_Variant_Results/')
parser.add_argument('--ref_file','-r', help="Required parameter, reference fasta file, run ./install.sh to dowload GRCh38 human reference fasta",required=True)

args = parser.parse_args()
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
__author__ = "Xin Zhou@Stanford"

def Split_supercontig_by_haplotype(fasta_file,xin):
    f = open(fasta_file,"r")
    output_prefix = fasta_file.split(".fasta")[0]
    hp1_file = output_prefix + "_hp1.fasta"
    hp2_file = output_prefix + "_hp2.fasta"
    fw_hp1 = open(hp1_file,"w")
    fw_hp2 = open(hp2_file,"w")
    count = 0
    for line in f:
        data = line.rsplit()
        if count%2 == 0:
            hp_flag = data[0].split("_")[-1]
            if hp_flag == "hp1":
                fw_hp1.writelines(line)
            elif hp_flag == "hp2":
                fw_hp2.writelines(line)
        elif count%2 == 1:
            if hp_flag == "hp1":
                fw_hp1.writelines(line)
            elif hp_flag == "hp2":
                fw_hp2.writelines(line)
        count += 1
    f.close()
    fw_hp1.close()
    fw_hp2.close()
    print("done")


def Split_haplotype(chr_start,chr_end,in_dir):
    pool = Pool(processes=chr_end - chr_start +1)
    for chr_num in range(chr_start,chr_end + 1):
        input_file = in_dir + "Aquila_Contig_chr" + str(chr_num) + ".fasta"
        pool.apply_async(Split_supercontig_by_haplotype,(input_file,"xin"))
    pool.close()
    while len(active_children()) > 1:
        time.sleep(0.5)
    pool.join()
    print("all done~")


def get_paf(ref_file,hap_file,hap_paf,xin):
    try:
        use_cmd =  "minimap2 -cx asm5 -t8 --cs " + ref_file + " " + hap_file  + " > " + hap_paf
    except:
        use_cmd =  code_path + "minimap2/" + "minimap2 -cx asm5 -t8 --cs " + ref_file + " " + hap_file  + " > " + hap_paf
    Popen(use_cmd,shell=True).wait()


def sort_paf(hap_paf,hap_paf_sorted,xin):
    use_cmd = "sort -k6,6 -k8,8n " + hap_paf + " > "  + hap_paf_sorted  
    Popen(use_cmd,shell=True).wait()


def get_var(hap_paf_sorted,hap_var_txt,xin):
    use_cmd = code_path + "/k8-0.2.4/k8-Linux "  + code_path + "paftools/" +  "/paftools.js " + " call -l 1 -L 1 -q 20 " +  hap_paf_sorted + " > " +  hap_var_txt 
    Popen(use_cmd,shell=True).wait()
    

def assembly_based_variants_call_paf(chr_start,chr_end,ref_file):
    for chr_num in range(chr_start,chr_end + 1):
        hap1_file = in_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.fasta"
        hap2_file = in_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.fasta"
        hap1_paf = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.paf"
        hap2_paf = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.paf"
        pool = Pool(processes=2)
        pool.apply_async(get_paf,(ref_file,hap1_file,hap1_paf,"xin"))
        pool.apply_async(get_paf,(ref_file,hap2_file,hap2_paf,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()

    print("all done~")


def assembly_based_variants_call_sort(chr_start,chr_end):
    for chr_num in range(chr_start,chr_end + 1):
        hap1_paf = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.paf"
        hap2_paf = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.paf"
        hap1_paf_sorted = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.paf.sorted" 
        hap2_paf_sorted = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.paf.sorted" 
        pool = Pool(processes=2)
        pool.apply_async(sort_paf,(hap1_paf,hap1_paf_sorted,"xin"))
        pool.apply_async(sort_paf,(hap2_paf,hap2_paf_sorted,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()

    print("all done~")
 

def assembly_based_variants_call(chr_start,chr_end):
    for chr_num in range(chr_start,chr_end + 1):
        hap1_paf_sorted = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.paf.sorted"
        hap2_paf_sorted = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.paf.sorted"
        hap1_var_txt = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp1.var.txt"
        hap2_var_txt = out_dir + "Aquila_Contig_chr" + str(chr_num) + "_hp2.var.txt"
        pool = Pool(processes=2)
        pool.apply_async(get_var,(hap1_paf_sorted,hap1_var_txt,"xin"))
        pool.apply_async(get_var,(hap2_paf_sorted,hap2_var_txt,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()

    print("all done~")


def Run_del_or_ins(all_cmd,xin):
    Popen(all_cmd,shell=True).wait()


def Call_SNV_info_from_contigs(chr_start,chr_end,out_dir,num_of_threads):
    total_num = chr_end - chr_start +1
    pool = Pool(num_of_threads)
    count = 1
    for chr_num in range(chr_start,chr_end + 1):
        all_cmd = "python3 " + code_path + "Extract_SNV_info_from_contigs_forcontiginfo_forall.py  --chr_start " + str(chr_num) + " --chr_end " + str(chr_num) + " --out_dir " + out_dir 
        count += 1
        pool.apply_async(Run_del_or_ins,(all_cmd,"xin"))
        if (count - 1)%num_of_threads == 0 or (count - 1) == total_num:
            pool.close()
            while len(active_children()) > 1:
                time.sleep(0.5)
            pool.join()

            if (count - 1) == total_num:
                print("finished chr" + str(chr_num))
            else:
                pool = Pool(num_of_threads)
    curr_file = out_dir + "Aquila_SNPs.vcf"
    exists = os.path.exists(curr_file)
    if exists:
        Popen("rm -rf " + curr_file,shell=True).wait()
    for chr_num in range(chr_start,chr_end + 1):
        one_vcf = out_dir + "Aquila_SNPs_chr" + str(chr_num) + ".vcf"
        cat_cmd = "cat " + one_vcf  + " >> " + out_dir + "Aquila_SNPs.vcf"
        Popen(cat_cmd,shell=True).wait()
    print("all done~")


def Call_SV_del_from_contigs(chr_start,chr_end,out_dir,num_of_threads,v_size):
    total_num = chr_end - chr_start +1
    pool = Pool(num_of_threads)
    count = 1
    for chr_num in range(chr_start,chr_end + 1):
        all_cmd = "python3 " + code_path + "Extract_SV_info_from_contigs_use_overlap_for_del_forcontiginfo.py --chr_start " + str(chr_num) + " --chr_end " + str(chr_num) + " --out_dir " + out_dir + " --var_size " + str(v_size)
        count += 1
        pool.apply_async(Run_del_or_ins,(all_cmd,"xin"))
        if (count - 1)%num_of_threads == 0 or (count - 1) == total_num:
            pool.close()
            while len(active_children()) > 1:
                time.sleep(0.5)
            pool.join()

            if (count - 1) == total_num:
                print("finished chr" + str(chr_num))
            else:
                pool = Pool(num_of_threads)
    curr_file = out_dir + "Aquila_DEL.vcf"
    exists = os.path.exists(curr_file)
    if exists:
        Popen("rm -rf " + curr_file,shell=True).wait()
    for chr_num in range(chr_start,chr_end + 1):
        one_vcf = out_dir + "Aquila_DEL_chr" + str(chr_num) + ".vcf"
        cat_cmd = "cat " + one_vcf  + " >> " + out_dir + "Aquila_DEL.vcf"
        Popen(cat_cmd,shell=True).wait()

    print("all done~")


def Call_SV_ins_from_contigs(chr_start,chr_end,out_dir,num_of_threads,v_size):
    total_num = chr_end - chr_start +1
    pool = Pool(num_of_threads)
    count = 1
    for chr_num in range(chr_start,chr_end + 1):
        all_cmd = "python3 " + code_path + "Extract_SV_info_from_contigs_use_shift_for_ins2_forcontiginfo.py --chr_start " + str(chr_num) + " --chr_end " + str(chr_num) + " --out_dir " + out_dir + " --var_size " + str(v_size)
        count += 1
        pool.apply_async(Run_del_or_ins,(all_cmd,"xin"))
        if (count - 1)%num_of_threads == 0 or (count - 1) == total_num:
            pool.close()
            while len(active_children()) > 1:
                time.sleep(0.5)
            pool.join()

            if (count - 1) == total_num:
                print("finished chr" + str(chr_num))
            else:
                pool = Pool(num_of_threads)
    curr_file = out_dir + "Aquila_INS.vcf"
    exists = os.path.exists(curr_file)
    if exists:
        Popen("rm -rf " + curr_file,shell=True).wait()
    for chr_num in range(chr_start,chr_end + 1):
        one_vcf = out_dir + "Aquila_INS_chr" + str(chr_num) + ".vcf"
        cat_cmd = "cat " + one_vcf  + " >> " + out_dir + "Aquila_INS.vcf"
        Popen(cat_cmd,shell=True).wait()

    print("all done~")



def Merge_all_variants(out_dir):
    snp_vcf = out_dir + "Aquila_SNPs.vcf"
    del_vcf = out_dir + "Aquila_DEL.vcf"
    ins_vcf = out_dir + "Aquila_INS.vcf"
    final_vcf = out_dir + "Aquila_final.vcf"
    final_sort_vcf = out_dir + "Aquila_final_sorted.vcf"
    use_cmd = "cat " + snp_vcf + " "  + del_vcf + " " + ins_vcf + " > " + final_vcf 
    sort_cmd = "cat " + final_vcf + " | awk '$1 ~ /^#/ {print $0;next} {print $0 | \"LC_ALL=C sort -k1,1 -k2,2n\"}' > " + final_sort_vcf
    Popen(use_cmd,shell=True).wait()
    Popen(sort_cmd,shell=True).wait()
    print("all done~")


def Clean_data(out_dir):
    rm_cmd = "rm " + out_dir + "/*chr*"
    Popen(rm_cmd,shell=True).wait()


def main():
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_assembly_based_variants_call.py -h",shell=True).wait()
    else:
        out_dir = args.out_dir + "/"
        if os.path.exists(out_dir):
            print("using existing output folder: " + out_dir)
        else:
            os.makedirs(out_dir)
        chr_start = args.chr_start
        chr_end = args.chr_end
        v_size = args.var_size
        num_of_threads = args.num_of_threads
        in_dir = args.assembly_dir + "/" + "Assembly_Contigs_files/"  
        ref_file = args.ref_file
        Split_haplotype(chr_start,chr_end,in_dir)
        assembly_based_variants_call_paf(chr_start,chr_end,ref_file)
        assembly_based_variants_call_sort(chr_start,chr_end)
        assembly_based_variants_call(chr_start,chr_end)
        Call_SNV_info_from_contigs(chr_start,chr_end,out_dir,num_of_threads)
        Call_SV_del_from_contigs(chr_start,chr_end,out_dir,num_of_threads,v_size)
        Call_SV_ins_from_contigs(chr_start,chr_end,out_dir,num_of_threads,v_size)
        Merge_all_variants(out_dir)
        Clean_data(out_dir)

if __name__ == "__main__":
	main()





#!/usr/bin/env python
import pdb
#pdb.set_trace()
from subprocess import Popen
from argparse import ArgumentParser
import os
import sys
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
__author__ = "Xin Zhou@Stanford"
parser = ArgumentParser(description="Author: xzhou15@cs.stanford.edu\n",usage='use "python3 %(prog)s --help" for more information')
parser.add_argument('--bam_file','-bam',help="Required parameter; BAM file, called by longranger align",required=True)
parser.add_argument('--vcf_file','-v',help="Required parameter; VCF file, called by FreeBayes",required=True)
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from, default = 1", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by,default = 23", default=23)
parser.add_argument('--sample_name','-name',help="Required parameter; Sample Name you can define by yourself, for example: S12878",required=True)
parser.add_argument('--out_dir','-o', help="Directory to store assembly results, default = ./Assembly_results",default="./Asssembly_results")
parser.add_argument('--uniq_map_dir','-uniq_dir', help="Required Parameter; Directory for 100-mer uniqness, run ./install to download \"Uniquess_map\" for hg38",required=True)
parser.add_argument('--num_threads','-t_chr',type=int,help="number of threads, default = 8 (recommended)", default=8)
parser.add_argument('--num_threads_for_samtools_sort','-t',type=int,help="number of threads for samtools sort, default = 20", default=20)
parser.add_argument('--num_threads_for_extract_reads','-t_extract',type=int,help="number of threads for extracting raw reads, default = 8 (recommended)", default=8)
parser.add_argument('--block_threshold','-bt',type=int,help="phase block threshold, default = 200000",default=200000)
parser.add_argument('--block_len_use','-bl',type=int,help="phase block len threshold, default = 100000",default=100000)
parser.add_argument('--mbq_threshold','-mbq',type=int,help="phred-scaled quality score for the assertion made in ALT, default = 13",default=13)
parser.add_argument('--boundary','-bound',type=int,help="boundary for long fragments with the same barcode, default = 50000 (recommended)",default=50000)

args = parser.parse_args()


def Get_fragment_files(bam_file,vcf_file,sample_name,chr_start,chr_end,h5_dir,num_threads,mbq,boundary):
    use_cmd = "python3 " + code_path + "Run_h5_all_multithreads.py" + " --bam_file " + bam_file + " --vcf_file " + vcf_file + " --sample_name " + sample_name + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --mbq " + str(mbq) + "  --mmq 0  --boundary " + str(boundary) + " --num_threads " + str(num_threads) + " --out_dir " + h5_dir
    Popen(use_cmd,shell=True).wait()


def Get_highconf_profile(bam_file,chr_start,chr_end,HighConf_file_dir,uniq_map_dir):
    use_cmd = "python3 " + code_path + "Generate_highconf_cut_profile_v2.py" + " --bam_file " +  bam_file + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --mbq 13 --mmq 20 " + " --out_dir " + HighConf_file_dir + " --uniq_map_dir " + uniq_map_dir
    Popen(use_cmd,shell=True).wait()
  

def Haplotying_fragments(chr_start,chr_end,phased_file_dir,h5_dir,sample_name):
    use_cmd = "python3 " + code_path + "Run_phase_alg_multithreads2.py" + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --overlap_threshold 3 --support_threshold 5 " + " --out_dir " + phased_file_dir  + " --h5_dir " + h5_dir + " --sample_name " + sample_name
    Popen(use_cmd,shell=True).wait()


def Cut_phase_blocks(chr_start,chr_end,block_threshold,block_len_use,phase_blocks_cut_highconf_dir,phased_file_dir,HighConf_file_dir):
    use_cmd = "python3 " + code_path + "Cut_phaseblock_for_phased_h5_v4.0_highconf_v2.py" + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --block_threshold " + str(block_threshold) + " --block_len_use " + str(block_len_use)  + " --out_dir " +  phase_blocks_cut_highconf_dir + " --in_dir " + phased_file_dir + " --highconf_profile_dir " + HighConf_file_dir
    Popen(use_cmd,shell=True).wait()


def Get_fastq_files_total(bam_file,chr_start,chr_end,num_threads,Raw_fastqs_dir,Sorted_bam_dir):
    use_cmd = "python3 " + code_path + "Read_fastqs_from_sortedbam_v2.py " + " --num_threads " + str(num_threads) + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --out_dir " + Raw_fastqs_dir + " --bam_file " + bam_file + " --bam_dir " + Sorted_bam_dir
    Popen(use_cmd,shell=True).wait()


def Extract_reads_for_small_chunks(chr_start,chr_end,h5_dir,phase_blocks_cut_highconf_dir,Local_Assembly_dir,Raw_fastqs_dir,block_len_use,sample_name,num_threads):
    use_cmd = "python3 " + code_path + "Run_extract_reads_for_smallchunks_all_lessmem.py" + " --phase_cut_folder " + phase_blocks_cut_highconf_dir + " --chr_start " + str(chr_start) + " --chr_end " + str(chr_end) + " --out_dir " + Local_Assembly_dir + " --h5_folder " + h5_dir +  " --fastq_folder " + Raw_fastqs_dir + " --cut_threshold " + str(block_len_use) + " --sample_name " + sample_name + " --num_threads " +   str(num_threads)
    Popen(use_cmd,shell=True).wait()




def main():
    if len(sys.argv) == 1:
        Popen("python3 " + "Aquila_step1.py -h",shell=True).wait()
    else:
        bam_file = args.bam_file
        vcf_file = args.vcf_file
        chr_start = args.chr_start
        chr_end = args.chr_end
        block_len_use = args.block_len_use
        block_threshold = args.block_threshold
        mbq = args.mbq_threshold
        boundary = args.boundary
        uniq_map_dir = args.uniq_map_dir + "/"
        num_threads = int(args.num_threads)
        num_threads_for_bwa_mem = int(args.num_threads_for_samtools_sort)
        num_threads_for_extract_reads = int(args.num_threads_for_extract_reads)
        sample_name = args.sample_name
        h5_dir = args.out_dir + "/H5_for_molecules/"
        HighConf_file_dir = args.out_dir + "/HighConf_file/"
        phased_file_dir = args.out_dir + "/results_phased_probmodel/"
        phase_blocks_cut_highconf_dir = args.out_dir + "/phase_blocks_cut_highconf/"
        Raw_fastqs_dir = args.out_dir + "/Raw_fastqs/"
        Sorted_bam_dir = args.out_dir + "/sorted_bam/"
        Local_Assembly_dir = args.out_dir + "/Local_Assembly_by_chunks/"
        
        Get_fragment_files(bam_file,vcf_file,sample_name,chr_start,chr_end,h5_dir,num_threads,mbq,boundary)
        Get_highconf_profile(bam_file,chr_start,chr_end,HighConf_file_dir,uniq_map_dir)
        Haplotying_fragments(chr_start,chr_end,phased_file_dir,h5_dir,sample_name)
        Cut_phase_blocks(chr_start,chr_end,block_threshold,block_len_use,phase_blocks_cut_highconf_dir,phased_file_dir,HighConf_file_dir)
        
        Get_fastq_files_total(bam_file,chr_start,chr_end,num_threads_for_bwa_mem,Raw_fastqs_dir,Sorted_bam_dir)
        Extract_reads_for_small_chunks(chr_start,chr_end,h5_dir,phase_blocks_cut_highconf_dir,Local_Assembly_dir,Raw_fastqs_dir,block_len_use,sample_name,num_threads_for_extract_reads)


if __name__ == "__main__":
	main()
    
    



#!/usr/bin/env python
import pdb
#pdb.set_trace()
import os
import numpy as np
import subprocess
from argparse import ArgumentParser
from multiprocessing import Pool,cpu_count,active_children
import time
import pickle
from collections import defaultdict
import sys
from subprocess import Popen
script_path = os.path.dirname(os.path.abspath( __file__ ))

code_path = script_path + "/" 

parser = ArgumentParser(description="Run depth all:")
parser.add_argument('--bam_file','-bam',help="input sorted bam file")
parser.add_argument('--chr_start','-start',type=int,help="chromosome start from", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chromosome end by", default=23)
parser.add_argument('--mbq','-bq',type=int,help="minimum base quality to consider a base for haplotype fragment, default 13", default=13)
parser.add_argument('--mmq','-mq',type=int,help="minimum read mapping quality to consider a read for phasing, default 20", default=20)
parser.add_argument('--out_dir','-o_dir', help="Directory to store outputs")
parser.add_argument('--uniq_map_dir','-uniq_dir', help="Directory to 100-mer uniqness")
args = parser.parse_args()


def get_read_depth(bam_file,mbq,mmq,chr_num,out_dir,xin):
    if chr_num == 23:
        chr_num = "X"
    else:
        chr_num = int(chr_num)
    noclips_bam = out_dir + "chr" + str(chr_num) + "_noclips.bam"
    depth_file_noHclip = out_dir + "chr" + str(chr_num) + "_depth_noHclip.txt"
    depth_file = out_dir  + "chr" + str(chr_num) + "_depth.txt"
    try:
        rm_hardclip_cmd = "samtools view -h " + bam_file + " chr" + str(chr_num) + " | awk '$6 !~ /H|S/{print}' | samtools view -bS - > " + noclips_bam
        depth_cmd = "samtools depth " + noclips_bam + " -q " + str(mbq_threshold) +  " -Q " + str(mmq_threshold) + " > " + depth_file_noHclip 
        depth_cmd_2 = "samtools depth " + bam_file + " -q " + str(mbq_threshold)  + " -Q " +str(mmq_threshold) + " -r chr" + str(chr_num) + " > " + depth_file 
    except:
        rm_hardclip_cmd = code_path + "samtools/" + "samtools view -h " + bam_file + " chr" + str(chr_num) + " | awk '$6 !~ /H|S/{print}' | " + code_path + "samtools/" + "samtools view -bS - > " + noclips_bam
        depth_cmd =  code_path + "samtools/" + "samtools depth " + noclips_bam + " -q " + str(mbq_threshold) +  " -Q " + str(mmq_threshold) + " > " + depth_file_noHclip 
        depth_cmd_2 = code_path + "samtools/" + "samtools depth " + bam_file + " -q " + str(mbq_threshold)  + " -Q " +str(mmq_threshold) + " -r chr" + str(chr_num) + " > " + depth_file 

    print(rm_hardclip_cmd)
    print(depth_cmd)
    print(depth_cmd_2)
    subprocess.Popen(rm_hardclip_cmd,shell=True).wait()
    subprocess.Popen(depth_cmd,shell=True).wait()
    subprocess.Popen(depth_cmd_2,shell=True).wait()



def get_coverage_per_pos(depth_file,output_file,xin):
    f = open(depth_file,"r")
    coverage_pos = defaultdict(int)
    for line in f:
        data = line.rsplit()
        pos_ = int(data[1]) + 1 # use "1" coordinates
        depth = int(data[2])
        coverage_pos[pos_] = depth
    print(len(coverage_pos))
    pickle.dump(coverage_pos,open(output_file,"wb"))


def get_global_track_for_breakpoints(cov_dict,uniq_map_dict,output_file,xin):
    global_track = defaultdict(list)
    cov_list = sorted(cov_dict.values())
    mean_cov = np.mean(cov_list)
    cov_threshold = mean_cov*0.8
    print(mean_cov, cov_threshold)
    for _pos, _cov in cov_dict.items():
        if _cov > cov_threshold:
            if _pos in uniq_map_dict:
                global_track[_pos] = [_cov]
    pickle.dump(global_track,open(output_file,"wb"))


def Run_all(bam_file,chr_start,chr_end,mbq_threshold,mmq_threshold,out_dir,uniq_map_dir):
    num_threads = 24
    round_times = int(np.ceil((chr_end - chr_start + 1)/num_threads))
    old_list = defaultdict(list)
    for ii in range(0,num_threads):
        for jj in range(chr_start + round_times*ii,chr_start + round_times*ii + round_times):
            if jj <= chr_end:
                old_list[ii+1].append(jj)

    new_list = defaultdict(list)
    times = 1
    for key,val in old_list.items():
        if times%2 == 1:
            new_list[key] = val
        elif times%2 == 0:
            new_list[key] = val[::-1] 
        times += 1
    
    run_list = defaultdict(list)
    for key,val in new_list.items():
        for ii in range(round_times):
            if ii < len(val):
                run_list[ii+1].append(val[ii])

    for key, val in run_list.items():
        pool = Pool(processes=num_threads)
        for chr_num in val:
            print("processing chr" + str(chr_num))
            pool.apply_async(get_read_depth,(bam_file,mbq_threshold,mmq_threshold,chr_num,out_dir,"xin"))
        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()
        print("done")

    ###########
    num_threads = 12
    round_times = int(np.ceil((chr_end - chr_start + 1)/num_threads))
    old_list = defaultdict(list)
    for ii in range(0,num_threads):
        for jj in range(chr_start + round_times*ii,chr_start + round_times*ii + round_times):
            if jj <= chr_end:
                old_list[ii+1].append(jj)

    new_list = defaultdict(list)
    times = 1
    for key,val in old_list.items():
        if times%2 == 1:
            new_list[key] = val
        elif times%2 == 0:
            new_list[key] = val[::-1] 
        times += 1
    
    run_list = defaultdict(list)
    for key,val in new_list.items():
        for ii in range(round_times):
            if ii < len(val):
                run_list[ii+1].append(val[ii])

    for key, val in run_list.items():
        pool = Pool(processes=num_threads)
        for chr_num in val:
            if chr_num == 23:
                depth_file = out_dir + "chrX_depth_noHclip.txt"
            else:
                depth_file = out_dir + "chr" + str(chr_num) + "_depth_noHclip.txt"
            output_file = out_dir + "chr" + str(chr_num) + "_pos_cf.p"
            pool.apply_async(get_coverage_per_pos,(depth_file,output_file,"xin"))

        pool.close()
        while len(active_children()) > 1:
            time.sleep(0.5)
        pool.join()
        print("done here")
    print("Extract all depth file, Done~")

   ###################
    for chr_num in range(chr_start,chr_end + 1):
        cov_file = out_dir + "chr" + str(chr_num) + "_pos_cf.p" 
        cov_dict = pickle.load(open(cov_file,"rb"))
        uniq_map_file = uniq_map_dir + "uniq_map_chr" + str(chr_num) + ".p"
        uniq_map_dict = pickle.load(open(uniq_map_file,"rb"))
        output_file = out_dir + "chr" + str(chr_num) + "_global_track.p"
        get_global_track_for_breakpoints(cov_dict,uniq_map_dict,output_file,"xin")
    print("Extract all depth file, Done~")


    print("Finishing generating highconf profile, all done~")

   


if __name__ == "__main__":
    out_dir = args.out_dir
    if os.path.exists(out_dir):
        print("using existing output folder: " + out_dir)
    else:
        os.makedirs(out_dir)
    bam_file = args.bam_file
    chr_start = args.chr_start
    chr_end = args.chr_end
    mbq_threshold = args.mbq
    mmq_threshold = args.mmq
    out_dir = args.out_dir
    uniq_map_dir = args.uniq_map_dir
    Run_all(bam_file,chr_start,chr_end,mbq_threshold,mmq_threshold,out_dir,uniq_map_dir)
    #Popen("rm " + out_dir + "*.bam",shell=True).wait()
    #Popen("rm " + out_dir + "*.txt",shell=True).wait()
    #Popen("rm " + out_dir + "*_pos_cf.p",shell=True).wait()



#!/usr/bin/env python
import pdb
#pdb.set_trace()
import pysam
from collections import defaultdict
from subprocess import Popen
from argparse import ArgumentParser
import os
import time
script_path = os.path.dirname(os.path.abspath( __file__ ))
code_path = script_path + "/" 
parser = ArgumentParser(description="extract fastq files from bam:")
parser.add_argument('--out_dir','-o', help="output folder")
parser.add_argument('--bam_dir','-bam_o', help="bam folder")
parser.add_argument('--bam_file','-bam', help="bam file")
parser.add_argument('--chr_start','-start',type=int,help="chr start", default=1)
parser.add_argument('--chr_end','-end',type=int,help="chr end", default=23)
parser.add_argument('--num_threads','-t',type=int,help="number of threads", default=20)
args = parser.parse_args()
out_dir = args.out_dir
bam_dir = args.bam_dir
chr_start = int(args.chr_start)
chr_end = int(args.chr_end)

if os.path.exists(out_dir):
    print("using existing output folder: " + out_dir)
else:
    os.makedirs(out_dir)

chr_dict = {"chr1":1,"chr2":2,"chr3":3,"chr4":4,"chr5":5,"chr6":6,"chr7":7,"chr8":8,"chr9":9,"chr10":10,"chr11":11,"chr12":12,"chr13":13,"chr14":14,"chr15":15,"chr16":16,"chr17":17,"chr18":18,"chr19":19,"chr20":20,"chr21":21,"chr22":22,"chrX":23}

fw_curr = defaultdict()
for chr_num in range(chr_start, chr_end+1):
    fw_curr[chr_num] = open(out_dir + "fastq_by_Chr_" + str(chr_num),"w")

tab = str.maketrans("ACTGN", "TGACN")

def reverse_complement(seq):
    return seq.translate(tab)[::-1]


def write_pair_reads(prev_read,read,use_chr_num):
    prev_read_seq = prev_read.seq
    prev_read_qual = prev_read.qual
    if prev_read.is_reverse:
        prev_read_seq = reverse_complement(prev_read_seq)
        prev_read_qual = prev_read_qual[::-1]
    read_seq = read.seq
    read_qual = read.qual
    if read.is_reverse:
        read_seq = reverse_complement(read_seq)
        read_qual = read_qual[::-1]
    fw_use = fw_curr[use_chr_num]
    if prev_read.is_read1:
        cur_line = "@" + prev_read.qname + "\n" + prev_read_seq + "\n" + "+\n" + prev_read_qual + "\n" + "@" + read.qname + "\n" + read_seq + "\n" + "+\n" + read_qual + "\n"
        fw_use.write(cur_line)
    elif prev_read.is_read2:
        cur_line = "@" + read.qname + "\n" + read_seq + "\n" + "+\n" + read_qual + "\n" + "@" + prev_read.qname + "\n" + prev_read_seq + "\n" + "+\n" + prev_read_qual + "\n"
        fw_use.write(cur_line)


def read_fastqs_from_sorted_bam(sorted_bam_file,chr_start,chr_end):
    sam_file = pysam.AlignmentFile(sorted_bam_file, "rb")
    count = 0
    count_unmapped = 0
    count_line = 0
    use_chr_dict = defaultdict(int)
    for ii in range(chr_start,chr_end + 1):
        use_chr_dict[ii] == 1
    for read in sam_file.fetch(until_eof=True):
        if not read.is_secondary:
            count += 1
            if count == 2:
                if prev_read.is_unmapped and read.is_unmapped:
                    count_unmapped += 1
                else:
                    chr_name_1 = prev_read.reference_name
                    chr_name_2 = read.reference_name
                    if chr_name_1 == chr_name_2:
                        if chr_name_1 in chr_dict:
                            use_chr_num = chr_dict[chr_name_1]
                            if use_chr_num in use_chr_dict:
                                write_pair_reads(prev_read,read,use_chr_num)
                    else:
                        if chr_name_1 in chr_dict:
                            use_chr_num = chr_dict[chr_name_1]
                            if use_chr_num in use_chr_dict:
                                write_pair_reads(prev_read,read,use_chr_num)
                        elif chr_name_2 in chr_dict:
                            use_chr_num = chr_dict[chr_name_2]
                            if use_chr_num in use_chr_dict:
                                write_pair_reads(prev_read,read,use_chr_num)
                count = 0
            if count == 1:
                prev_read = read


def close_file(chr_start,chr_end):
    for chr_num in range(chr_start, chr_end+1):
        fw_curr[chr_num].close()




if __name__ == "__main__":
    bam_file = args.bam_file
    num_threads = int(args.num_threads)
    bam_sorted_idx_file = bam_dir + "finish_bam.txt"
    bam_sorted_file = bam_dir + "sorted_bam.bam"
    if os.path.exists(bam_dir):
        print("using existing output folder: " + bam_dir)
        while not os.path.exists(bam_sorted_idx_file):
            time.sleep(1)
    else:
        os.makedirs(bam_dir)
        try:
            sort_bam_cmd = "samtools sort -@ " + str(num_threads) + " -n " + bam_file + " -o " + bam_sorted_file
        except:
            sort_bam_cmd = code_path + "samtools/" + "samtools sort -@ " + str(num_threads) + " -n " + bam_file + " -o " + bam_sorted_file

        Popen(sort_bam_cmd,shell=True).wait()
        Popen("touch " + bam_dir + "finish_bam.txt",shell=True).wait()

    read_fastqs_from_sorted_bam(bam_sorted_file,chr_start,chr_end)
    close_file(chr_start,chr_end)




import pdb
#pdb.set_trace()
from collections import defaultdict
import pickle
import os
from argparse import ArgumentParser

def save_pickle_file(dict1,fname):
    for value in dict1:
        dict1[value] = dict(dict1[value])
    my_dict = dict(dict1)
    with open(fname, "wb") as f:
        pickle.dump(my_dict,f) 


def Extract_mole_num_from_phased_mole(phased_file,PS_flag_dict_cut_file,chr_num,mole_num_start,mole_num_end,mole_num_add):
    phase_dict = defaultdict(list)
    PS_flag_dict_cut = pickle.load(open(PS_flag_dict_cut_file,"rb"))
    f = open(phased_file,"r")
    for line in f:
        data = line.rsplit()
        PS_flag_info = data[-2]
        PS_flag = int(PS_flag_info.split(":")[1])
        HP_flag = data[-1]
        mole_num = int(data[6])
        if mole_num <= mole_num_end + mole_num_add and mole_num >= mole_num_start + mole_num_add: 
            PS_flag_end = PS_flag_dict_cut[PS_flag]
            phase_dict[(PS_flag,PS_flag_end,HP_flag)].append(mole_num - mole_num_add)

    #pickle.dump(phase_dict,open("phase_dict_chr" + str(chr_num) + ".p","wb"))
    print("done~")
    return phase_dict

def check_qname_in_PS(block_info,qname_pos_list):
    block_start = block_info[0]
    block_end = block_info[1]
    flag_in = 0
    for pos in qname_pos_list:
        if pos <= block_end and pos >= block_start:
            flag_in = 1
            break
    return flag_in 


def Extract_qname(phased_dict_mole_num,mole_qname_dict,qname_pos_dict,barcoded_fastq_file,chr_num,output_dir):
    phased_dict_qname_2 = defaultdict(lambda: defaultdict(int))
    qname_total_dict = defaultdict(int)
    curr = 0
    flag_in = 0
    for key, mole_num_list in phased_dict_mole_num.items():
        curr += 1
        for mole_num in mole_num_list:
            qname_list = mole_qname_dict[mole_num]
            for qname in qname_list:
                flag_in = check_qname_in_PS(key,qname_pos_dict[qname])
                if flag_in == 1:
                    phased_dict_qname_2[qname][key] = 1
    #save_pickle_file(phased_dict_qname_2,"phased_dict_qname_2.p")
    
    f = open(barcoded_fastq_file,"r")
    count = 0
    flag = 0
    for line in f:
        data = line.rsplit()
        if count%8 == 0:
            qname_curr = data[0][1:]
            if phased_dict_qname_2[qname_curr] != {}:
                flag = 1
                PS_flag_info = phased_dict_qname_2[qname_curr]
                for _PS_HP, _val in PS_flag_info.items():
                    _PS_flag = _PS_HP[0]
                    _PS_flag_end = _PS_HP[1]
                    _HP_flag = _PS_HP[2]
                file_curr = output_dir +"fastq_by_" + str(_PS_flag) + "_" + str(_PS_flag_end) + "_" + _HP_flag + ".fastq"
                if os.path.isfile(file_curr):
                    fw_curr = open(file_curr,"a")
                    fw_curr.write(line)
                else:
                    fw_curr = open(file_curr,"w")
                    fw_curr.write(line)
                del phased_dict_qname_2[qname_curr]

            else:
                del phased_dict_qname_2[qname_curr]

        elif count%8 == 7:
            if flag == 1:
                flag = 0
                fw_curr.write(line)
                fw_curr.close()

        else:
            if flag == 1:
                fw_curr.write(line)

        count += 1

    print("finished extracting...")



def Extract_start(output_dir,chr_num,phased_h5_file,PS_flag_dict_cut_file,mole_qname_dict_file,qname_pos_dict_file,chr_fastq,mole_num_start,mole_num_end,mole_num_add,xin):
    if os.path.exists(output_dir):
        print("using existing output folder: " + output_dir)
    else:
        os.makedirs(output_dir)
    phased_dict_mole_num = Extract_mole_num_from_phased_mole(phased_h5_file,PS_flag_dict_cut_file,chr_num,mole_num_start,mole_num_end,mole_num_add)
    mole_qname_dict = pickle.load(open(mole_qname_dict_file,"rb"))
    qname_pos_dict = pickle.load(open(qname_pos_dict_file,"rb"))
    Extract_qname(phased_dict_mole_num,mole_qname_dict,qname_pos_dict,chr_fastq,chr_num,output_dir)
